@@ -1,6 +1,9 @@
-from graphql import graphql_sync
+from collections import ChainMap
+from typing import Any
+
 from graphql.error import GraphQLError
-from graphql.language import SourceLocation
+from graphql.execution import execute_sync, ExecutionResult
+from graphql.language import parse, SourceLocation
 from graphql.type import (
     GraphQLArgument,
     GraphQLField,
@@ -14,61 +17,91 @@ from graphql.type import (
 
 
 def describe_execute_resolve_function():
-    def _test_schema(test_field):
+    def _test_schema(test_field: GraphQLField) -> GraphQLSchema:
         return GraphQLSchema(GraphQLObjectType("Query", {"test": test_field}))
 
     def default_function_accesses_attributes():
-        schema = _test_schema(GraphQLField(GraphQLString))
-
-        class Source:
+        class RootValue:
             test = "testValue"
 
-        assert graphql_sync(schema, "{ test }", Source()) == (
+        assert execute_sync(
+            schema=_test_schema(GraphQLField(GraphQLString)),
+            document=parse("{ test }"),
+            root_value=RootValue(),
+        ) == (
             {"test": "testValue"},
             None,
         )
 
-    def default_function_accesses_keys():
-        schema = _test_schema(GraphQLField(GraphQLString))
+    def default_function_accesses_keys_of_dict():
+        root_value = {"test": "testValue"}
 
-        source = {"test": "testValue"}
+        assert (
+            execute_sync(
+                schema=_test_schema(GraphQLField(GraphQLString)),
+                document=parse("{ test }"),
+                root_value=root_value,
+            )
+            == ({"test": "testValue"}, None)
+        )
 
-        assert graphql_sync(schema, "{ test }", source) == ({"test": "testValue"}, None)
+    def default_function_accesses_keys_of_chain_map():
+        # use a mapping that is not a subclass of dict
+        root_value = ChainMap({"test": "testValue"})
+
+        assert (
+            execute_sync(
+                schema=_test_schema(GraphQLField(GraphQLString)),
+                document=parse("{ test }"),
+                root_value=root_value,
+            )
+            == ({"test": "testValue"}, None)
+        )
 
     def default_function_calls_methods():
-        schema = _test_schema(GraphQLField(GraphQLString))
-
-        class Source:
-            _secret = "testValue"
+        class RootValue:
+            _secret = "secretValue"
 
             def test(self, _info):
                 return self._secret
 
-        assert graphql_sync(schema, "{ test }", Source()) == (
-            {"test": "testValue"},
+        assert execute_sync(
+            schema=_test_schema(GraphQLField(GraphQLString)),
+            document=parse("{ test }"),
+            root_value=RootValue(),
+        ) == (
+            {"test": "secretValue"},
             None,
         )
 
     def default_function_passes_args_and_context():
-        schema = _test_schema(
-            GraphQLField(GraphQLInt, args={"addend1": GraphQLArgument(GraphQLInt)})
-        )
-
         class Adder:
             _num: int
 
             def __init__(self, num):
                 self._num = num
 
-            def test(self, info, addend1):
+            def test(self, info, addend1: int):
                 return self._num + addend1 + info.context.addend2
 
-        source = Adder(700)
+        root_value = Adder(700)
 
-        class Context:
+        schema = _test_schema(
+            GraphQLField(GraphQLInt, args={"addend1": GraphQLArgument(GraphQLInt)})
+        )
+
+        class ContextValue:
             addend2 = 9
 
-        assert graphql_sync(schema, "{ test(addend1: 80) }", source, Context()) == (
+        context_value = ContextValue()
+        document = parse("{ test(addend1: 80) }")
+
+        assert execute_sync(
+            schema=schema,
+            document=document,
+            root_value=root_value,
+            context_value=context_value,
+        ) == (
             {"test": 789},
             None,
         )
@@ -85,21 +118,30 @@ def describe_execute_resolve_function():
             )
         )
 
-        assert graphql_sync(schema, "{ test }") == ({"test": "[None, {}]"}, None)
+        def execute_query(query: str, root_value: Any = None) -> ExecutionResult:
+            document = parse(query)
+            return execute_sync(
+                schema=schema,
+                document=document,
+                root_value=root_value,
+            )
 
-        assert graphql_sync(schema, "{ test }", "Source!") == (
+        assert execute_query("{ test }") == ({"test": "[None, {}]"}, None)
+
+        assert execute_query("{ test }", "Source!") == (
             {"test": "['Source!', {}]"},
             None,
         )
 
-        assert graphql_sync(schema, '{ test(aStr: "String!") }', "Source!") == (
+        assert execute_query('{ test(aStr: "String!") }', "Source!") == (
             {"test": "['Source!', {'aStr': 'String!'}]"},
             None,
         )
 
-        assert graphql_sync(
-            schema, '{ test(aInt: -123, aStr: "String!") }', "Source!"
-        ) == ({"test": "['Source!', {'aStr': 'String!', 'aInt': -123}]"}, None)
+        assert execute_query('{ test(aInt: -123, aStr: "String!") }', "Source!") == (
+            {"test": "['Source!', {'aStr': 'String!', 'aInt': -123}]"},
+            None,
+        )
 
     def transforms_arguments_using_out_names():
         # This is an extension of GraphQL.js.
@@ -114,21 +156,26 @@ def describe_execute_resolve_function():
             )
         )
 
-        assert graphql_sync(schema, "{ test }") == ({"test": "[None, {}]"}, None)
+        def execute_query(query: str, root_value: Any = None) -> ExecutionResult:
+            document = parse(query)
+            return execute_sync(schema=schema, document=document, root_value=root_value)
 
-        assert graphql_sync(schema, "{ test }", "Source!") == (
+        assert execute_query("{ test }") == ({"test": "[None, {}]"}, None)
+
+        assert execute_query("{ test }", "Source!") == (
             {"test": "['Source!', {}]"},
             None,
         )
 
-        assert graphql_sync(schema, '{ test(aStr: "String!") }', "Source!") == (
+        assert execute_query('{ test(aStr: "String!") }', "Source!") == (
             {"test": "['Source!', {'a_str': 'String!'}]"},
             None,
         )
 
-        assert graphql_sync(
-            schema, '{ test(aInt: -123, aStr: "String!") }', "Source!"
-        ) == ({"test": "['Source!', {'a_str': 'String!', 'a_int': -123}]"}, None)
+        assert execute_query('{ test(aInt: -123, aStr: "String!") }', "Source!") == (
+            {"test": "['Source!', {'a_str': 'String!', 'a_int': -123}]"},
+            None,
+        )
 
     def transforms_arguments_with_inputs_using_out_names():
         # This is an extension of GraphQL.js.
@@ -150,14 +197,18 @@ def describe_execute_resolve_function():
             )
         )
 
-        assert graphql_sync(schema, "{ test }") == ({"test": "[None, {}]"}, None)
+        def execute_query(query: str, root_value: Any = None) -> ExecutionResult:
+            document = parse(query)
+            return execute_sync(schema=schema, document=document, root_value=root_value)
 
-        assert graphql_sync(
-            schema, '{ test(aInput: {inputOne: "String!"}) }', "Source!"
-        ) == ({"test": "['Source!', {'a_input': {'input_one': 'String!'}}]"}, None)
+        assert execute_query("{ test }") == ({"test": "[None, {}]"}, None)
 
-        assert graphql_sync(
-            schema,
+        assert execute_query('{ test(aInput: {inputOne: "String!"}) }', "Source!") == (
+            {"test": "['Source!', {'a_input': {'input_one': 'String!'}}]"},
+            None,
+        )
+
+        assert execute_query(
             '{ test(aInput: {inputRecursive: {inputOne: "SourceRecursive!"}}) }',
             "Source!",
         ) == (
@@ -173,7 +224,7 @@ def describe_execute_resolve_function():
             raise ValueError("Some error")
 
         schema = _test_schema(GraphQLField(GraphQLString, resolve=resolve))
-        result = graphql_sync(schema, "{ test }")
+        result = execute_sync(schema, parse("{ test }"))
 
         assert result == (
             {"test": None},
