@@ -1,6 +1,6 @@
 from typing import Optional, Union, List
 
-from graphql.execution import execute
+from graphql.execution import execute_sync
 from graphql.language import parse
 from graphql.type import (
     GraphQLBoolean,
@@ -108,9 +108,9 @@ CatType = GraphQLObjectType(
 
 def resolve_pet_type(value, _info, _type):
     if isinstance(value, Dog):
-        return DogType
+        return DogType.name
     if isinstance(value, Cat):
-        return CatType
+        return CatType.name
 
     # Not reachable. All possible types have been considered.
     raise TypeError("Unexpected pet type")
@@ -182,7 +182,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document) == (
+        assert execute_sync(schema=schema, document=document) == (
             {
                 "Named": {
                     "kind": "INTERFACE",
@@ -244,7 +244,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -278,7 +278,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -307,7 +307,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -354,7 +354,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -399,7 +399,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -459,7 +459,7 @@ def describe_execute_union_and_intersection_types():
             """
         )
 
-        assert execute(schema=schema, document=document, root_value=john) == (
+        assert execute_sync(schema=schema, document=document, root_value=john) == (
             {
                 "__typename": "Person",
                 "name": "John",
@@ -493,7 +493,7 @@ def describe_execute_union_and_intersection_types():
             encountered["context"] = info.context
             encountered["schema"] = info.schema
             encountered["root_value"] = info.root_value
-            return PersonType2
+            return PersonType2.name
 
         NamedType2 = GraphQLInterfaceType(
             "Named", {"name": GraphQLField(GraphQLString)}, resolve_type=resolve_type
@@ -513,12 +513,15 @@ def describe_execute_union_and_intersection_types():
         root_value = Person("John", [], [liz])
         context_value = {"authToken": "123abc"}
 
-        assert execute(
+        assert execute_sync(
             schema=schema2,
             document=document,
             root_value=root_value,
             context_value=context_value,
-        ) == ({"name": "John", "friends": [{"name": "Liz"}]}, None,)
+        ) == (
+            {"name": "John", "friends": [{"name": "Liz"}]},
+            None,
+        )
 
         assert encountered == {
             "schema": schema2,
