@@ -1,6 +1,6 @@
-from typing import List, Optional
+from typing import List, Optional, Tuple
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql.error import GraphQLSyntaxError
 from graphql.language import Lexer, Source, SourceLocation, Token, TokenKind
@@ -9,6 +9,8 @@ from graphql.pyutils import inspect
 
 from ..utils import dedent
 
+Location = Optional[Tuple[int, int]]
+
 
 def lex_one(s: str) -> Token:
     lexer = Lexer(Source(s))
@@ -21,7 +23,7 @@ def lex_second(s: str) -> Token:
     return lexer.advance()
 
 
-def assert_syntax_error(text, message, location):
+def assert_syntax_error(text: str, message: str, location: Location) -> None:
     with raises(GraphQLSyntaxError) as exc_info:
         lex_second(text)
     error = exc_info.value
