@@ -1,8 +1,7 @@
-from typing import cast
+from typing import cast, Optional, Tuple
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
-from graphql.pyutils import dedent, inspect
 from graphql.error import GraphQLSyntaxError
 from graphql.language import (
     ArgumentNode,
@@ -28,48 +27,34 @@ from graphql.language import (
     parse_value,
     Source,
 )
+from graphql.pyutils import inspect
 
-# noinspection PyUnresolvedReferences
 from ..fixtures import kitchen_sink_query  # noqa: F401
+from ..utils import dedent
+
+Location = Optional[Tuple[int, int]]
 
 
-def assert_syntax_error(text, message, location):
+def assert_syntax_error(text: str, message: str, location: Location) -> None:
     with raises(GraphQLSyntaxError) as exc_info:
         parse(text)
     error = exc_info.value
-    assert message in error.message
+    assert error.message == f"Syntax Error: {message}"
+    assert error.description == message
     assert error.locations == [location]
 
 
 def describe_parser():
-    def asserts_that_a_source_to_parse_was_provided():
-        with raises(TypeError) as exc_info:
-            # noinspection PyArgumentList
-            assert parse()  # type: ignore
-        msg = str(exc_info.value)
-        assert "missing" in msg
-        assert "source" in msg
-        with raises(TypeError) as exc_info:
-            # noinspection PyTypeChecker
-            assert parse(None)  # type: ignore
-        msg = str(exc_info.value)
-        assert "Must provide Source. Received: None" in msg
-        with raises(TypeError) as exc_info:
-            # noinspection PyTypeChecker
-            assert parse({})  # type: ignore
-        msg = str(exc_info.value)
-        assert "Must provide Source. Received: {}" in msg
-
     def parse_provides_useful_errors():
         with raises(GraphQLSyntaxError) as exc_info:
             parse("{")
         error = exc_info.value
-        assert error.message == "Syntax Error: Expected Name, found <EOF>"
+        assert error.message == "Syntax Error: Expected Name, found <EOF>."
         assert error.positions == [1]
         assert error.locations == [(1, 2)]
         assert str(error) + "\n" == dedent(
             """
-            Syntax Error: Expected Name, found <EOF>
+            Syntax Error: Expected Name, found <EOF>.
 
             GraphQL request:1:2
             1 | {
@@ -78,14 +63,15 @@ def describe_parser():
         )
         assert_syntax_error(
             "\n      { ...MissingOn }\n      fragment MissingOn Type",
-            "Expected 'on', found Name 'Type'",
+            "Expected 'on', found Name 'Type'.",
             (3, 26),
         )
-        assert_syntax_error("{ field: {} }", "Expected Name, found {", (1, 10))
+        assert_syntax_error("{ field: {} }", "Expected Name, found '{'.", (1, 10))
         assert_syntax_error(
-            "notanoperation Foo { field }", "Unexpected Name 'notanoperation'", (1, 1)
+            "notAnOperation Foo { field }", "Unexpected Name 'notAnOperation'.", (1, 1)
         )
-        assert_syntax_error("...", "Unexpected ...", (1, 1))
+        assert_syntax_error("...", "Unexpected '...'.", (1, 1))
+        assert_syntax_error('{ ""', "Expected Name, found String ''.", (1, 3))
 
     def parse_provides_useful_error_when_using_source():
         with raises(GraphQLSyntaxError) as exc_info:
@@ -93,7 +79,7 @@ def describe_parser():
         error = exc_info.value
         assert str(error) + "\n" == dedent(
             """
-            Syntax Error: Expected {, found <EOF>
+            Syntax Error: Expected '{', found <EOF>.
 
             MyQuery.graphql:1:6
             1 | query
@@ -107,7 +93,7 @@ def describe_parser():
     def parses_constant_default_values():
         assert_syntax_error(
             "query Foo($x: Complex = { a: { b: [ $var ] } }) { field }",
-            "Unexpected $",
+            "Unexpected '$'.",
             (1, 37),
         )
 
@@ -115,10 +101,12 @@ def describe_parser():
         parse("query Foo($x: Boolean = false @bar) { field }")
 
     def does_not_accept_fragments_named_on():
-        assert_syntax_error("fragment on on on { on }", "Unexpected Name 'on'", (1, 10))
+        assert_syntax_error(
+            "fragment on on on { on }", "Unexpected Name 'on'.", (1, 10)
+        )
 
     def does_not_accept_fragments_spread_of_on():
-        assert_syntax_error("{ ...on }", "Expected Name, found }", (1, 9))
+        assert_syntax_error("{ ...on }", "Expected Name, found '}'.", (1, 9))
 
     def parses_multi_byte_characters():
         # Note: \u0A0A could be naively interpreted as two line-feed chars.
