@@ -1,9 +1,11 @@
 from textwrap import dedent
+from typing import List, Optional, Tuple
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql.error import GraphQLSyntaxError
 from graphql.language import (
+    ArgumentNode,
     BooleanValueNode,
     DirectiveDefinitionNode,
     DirectiveNode,
@@ -14,6 +16,7 @@ from graphql.language import (
     InputObjectTypeDefinitionNode,
     InputValueDefinitionNode,
     InterfaceTypeDefinitionNode,
+    InterfaceTypeExtensionNode,
     ListTypeNode,
     NameNode,
     NamedTypeNode,
@@ -23,25 +26,30 @@ from graphql.language import (
     OperationType,
     OperationTypeDefinitionNode,
     ScalarTypeDefinitionNode,
+    SchemaDefinitionNode,
     SchemaExtensionNode,
     StringValueNode,
+    TypeNode,
     UnionTypeDefinitionNode,
+    ValueNode,
     parse,
 )
 
-# noinspection PyUnresolvedReferences
 from ..fixtures import kitchen_sink_sdl  # noqa: F401
 
+Location = Optional[Tuple[int, int]]
 
-def assert_syntax_error(text, message, location):
+
+def assert_syntax_error(text: str, message: str, location: Location) -> None:
     with raises(GraphQLSyntaxError) as exc_info:
         parse(text)
     error = exc_info.value
-    assert message in error.message
+    assert error.message == f"Syntax Error: {message}"
+    assert error.description == message
     assert error.locations == [location]
 
 
-def assert_definitions(body, loc, num=1):
+def assert_definitions(body: str, loc: Location, num=1):
     doc = parse(body)
     assert isinstance(doc, DocumentNode)
     assert doc.loc == loc
@@ -51,35 +59,37 @@ def assert_definitions(body, loc, num=1):
     return definitions[0] if num == 1 else definitions
 
 
-def type_node(name, loc):
+def type_node(name: str, loc: Location):
     return NamedTypeNode(name=name_node(name, loc), loc=loc)
 
 
-def name_node(name, loc):
+def name_node(name: str, loc: Location):
     return NameNode(value=name, loc=loc)
 
 
-def field_node(name, type_, loc):
+def field_node(name: NameNode, type_: TypeNode, loc: Location):
     return field_node_with_args(name, type_, [], loc)
 
 
-def field_node_with_args(name, type_, args, loc):
+def field_node_with_args(name: NameNode, type_: TypeNode, args: List, loc: Location):
     return FieldDefinitionNode(
         name=name, arguments=args, type=type_, directives=[], loc=loc, description=None
     )
 
 
-def non_null_type(type_, loc):
+def non_null_type(type_: TypeNode, loc: Location):
     return NonNullTypeNode(type=type_, loc=loc)
 
 
-def enum_value_node(name, loc):
+def enum_value_node(name: str, loc: Location):
     return EnumValueDefinitionNode(
         name=name_node(name, loc), directives=[], loc=loc, description=None
     )
 
 
-def input_value_node(name, type_, default_value, loc):
+def input_value_node(
+    name: NameNode, type_: TypeNode, default_value: Optional[ValueNode], loc: Location
+):
     return InputValueDefinitionNode(
         name=name,
         type=type_,
@@ -90,25 +100,33 @@ def input_value_node(name, type_, default_value, loc):
     )
 
 
-def boolean_value_node(value, loc):
+def boolean_value_node(value: bool, loc: Location):
     return BooleanValueNode(value=value, loc=loc)
 
 
-def list_type_node(type_, loc):
+def string_value_node(value: str, block: Optional[bool], loc: Location):
+    return StringValueNode(value=value, block=block, loc=loc)
+
+
+def list_type_node(type_: TypeNode, loc: Location):
     return ListTypeNode(type=type_, loc=loc)
 
 
-def schema_extension_node(directives, operation_types, loc):
+def schema_extension_node(
+    directives: List[DirectiveNode],
+    operation_types: List[OperationTypeDefinitionNode],
+    loc: Location,
+):
     return SchemaExtensionNode(
         directives=directives, operation_types=operation_types, loc=loc
     )
 
 
-def operation_type_definition(operation, type_, loc):
+def operation_type_definition(operation: OperationType, type_: TypeNode, loc: Location):
     return OperationTypeDefinitionNode(operation=operation, type=type_, loc=loc)
 
 
-def directive_node(name, arguments, loc):
+def directive_node(name: NameNode, arguments: List[ArgumentNode], loc: Location):
     return DirectiveNode(name=name, arguments=arguments, loc=loc)
 
 
@@ -147,10 +165,7 @@ def describe_schema_parser():
         assert isinstance(definition, ObjectTypeDefinitionNode)
         assert definition.name == name_node("Hello", (20, 25))
         description = definition.description
-        assert isinstance(description, StringValueNode)
-        assert description.value == "Description"
-        assert description.block is False
-        assert description.loc == (1, 14)
+        assert description == string_value_node("Description", False, (1, 14))
 
     def parses_type_with_description_multi_line_string():
         body = dedent(
@@ -167,10 +182,24 @@ def describe_schema_parser():
         assert isinstance(definition, ObjectTypeDefinitionNode)
         assert definition.name == name_node("Hello", (60, 65))
         description = definition.description
-        assert isinstance(description, StringValueNode)
-        assert description.value == "Description"
-        assert description.block is True
-        assert description.loc == (1, 20)
+        assert description == string_value_node("Description", True, (1, 20))
+
+    def parses_schema_with_description_string():
+        body = dedent(
+            """
+            "Description"
+            schema {
+              query: Foo
+            }
+            """
+        )
+        definition = assert_definitions(body, (0, 39))
+        assert isinstance(definition, SchemaDefinitionNode)
+        description = definition.description
+        assert description == string_value_node("Description", False, (1, 14))
+
+    def description_followed_by_something_other_than_type_system_definition_throws():
+        assert_syntax_error('"Description" 1', "Unexpected Int '1'.", (1, 15))
 
     def simple_extension():
         body = dedent(
@@ -192,7 +221,7 @@ def describe_schema_parser():
         ]
         assert extension.loc == (1, 38)
 
-    def extension_without_fields():
+    def object_extension_without_fields():
         body = "extend type Hello implements Greeting"
         extension = assert_definitions(body, (0, 37))
         assert isinstance(extension, ObjectTypeExtensionNode)
@@ -202,7 +231,17 @@ def describe_schema_parser():
         assert extension.fields == []
         assert extension.loc == (0, 37)
 
-    def extension_without_fields_followed_by_extension():
+    def interface_extension_without_fields():
+        body = "extend interface Hello implements Greeting"
+        extension = assert_definitions(body, (0, 42))
+        assert isinstance(extension, InterfaceTypeExtensionNode)
+        assert extension.name == name_node("Hello", (17, 22))
+        assert extension.interfaces == [type_node("Greeting", (34, 42))]
+        assert extension.directives == []
+        assert extension.fields == []
+        assert extension.loc == (0, 42)
+
+    def object_extension_without_fields_followed_by_extension():
         body = (
             "\n      extend type Hello implements Greeting\n\n"
             "      extend type Hello implements SecondGreeting\n    "
@@ -224,16 +263,42 @@ def describe_schema_parser():
         assert extension.loc == (52, 95)
 
     def extension_without_anything_throws():
-        assert_syntax_error("extend type Hello", "Unexpected <EOF>", (1, 18))
+        assert_syntax_error("extend scalar Hello", "Unexpected <EOF>.", (1, 20))
+        assert_syntax_error("extend type Hello", "Unexpected <EOF>.", (1, 18))
+        assert_syntax_error("extend interface Hello", "Unexpected <EOF>.", (1, 23))
+        assert_syntax_error("extend union Hello", "Unexpected <EOF>.", (1, 19))
+        assert_syntax_error("extend enum Hello", "Unexpected <EOF>.", (1, 18))
+        assert_syntax_error("extend input Hello", "Unexpected <EOF>.", (1, 19))
 
-    def extension_do_not_include_descriptions():
+    def interface_extension_without_fields_followed_by_extension():
+        body = (
+            "\n      extend interface Hello implements Greeting\n\n"
+            "      extend interface Hello implements SecondGreeting\n    "
+        )
+        extensions = assert_definitions(body, (0, 110), 2)
+        extension = extensions[0]
+        assert isinstance(extension, InterfaceTypeExtensionNode)
+        assert extension.name == name_node("Hello", (24, 29))
+        assert extension.interfaces == [type_node("Greeting", (41, 49))]
+        assert extension.directives == []
+        assert extension.fields == []
+        assert extension.loc == (7, 49)
+        extension = extensions[1]
+        assert isinstance(extension, InterfaceTypeExtensionNode)
+        assert extension.name == name_node("Hello", (74, 79))
+        assert extension.interfaces == [type_node("SecondGreeting", (91, 105))]
+        assert extension.directives == []
+        assert extension.fields == []
+        assert extension.loc == (57, 105)
+
+    def object_extension_do_not_include_descriptions():
         assert_syntax_error(
             """
             "Description"
             extend type Hello {
               world: String
             }""",
-            "Unexpected Name 'extend'",
+            "Unexpected Name 'extend'.",
             (3, 13),
         )
         assert_syntax_error(
@@ -241,7 +306,26 @@ def describe_schema_parser():
             extend "Description" type Hello {
               world: String
             }""",
-            "Unexpected String 'Description'",
+            "Unexpected String 'Description'.",
+            (2, 20),
+        )
+
+    def interface_extension_do_not_include_descriptions():
+        assert_syntax_error(
+            """
+            "Description"
+            extend interface Hello {
+              world: String
+            }""",
+            "Unexpected Name 'extend'.",
+            (3, 13),
+        )
+        assert_syntax_error(
+            """
+            extend "Description" interface Hello {
+              world: String
+            }""",
+            "Unexpected String 'Description'.",
             (2, 20),
         )
 
@@ -281,7 +365,12 @@ def describe_schema_parser():
         ]
 
     def schema_extension_without_anything_throws():
-        assert_syntax_error("extend schema", "Unexpected <EOF>", (1, 14))
+        assert_syntax_error("extend schema", "Unexpected <EOF>.", (1, 14))
+
+    def schema_extension_with_invalid_operation_type_throws():
+        assert_syntax_error(
+            "extend schema { unknown: SomeType }", "Unexpected Name 'unknown'.", (1, 17)
+        )
 
     def simple_non_null_type():
         body = dedent(
@@ -306,6 +395,21 @@ def describe_schema_parser():
         ]
         assert definition.loc == (1, 32)
 
+    def simple_interface_inheriting_interface():
+        body = "interface Hello implements World { field: String }"
+        definition = assert_definitions(body, (0, 50))
+        assert isinstance(definition, InterfaceTypeDefinitionNode)
+        assert definition.name == name_node("Hello", (10, 15))
+        assert definition.description is None
+        assert definition.interfaces == [type_node("World", (27, 32))]
+        assert definition.directives == []
+        assert definition.fields == [
+            field_node(
+                name_node("field", (35, 40)), type_node("String", (42, 48)), (35, 48)
+            )
+        ]
+        assert definition.loc == (0, 50)
+
     def simple_type_inheriting_interface():
         body = "type Hello implements World { field: String }"
         definition = assert_definitions(body, (0, 45))
@@ -339,6 +443,24 @@ def describe_schema_parser():
         ]
         assert definition.loc == (0, 48)
 
+    def simple_interface_inheriting_multiple_interfaces():
+        body = "interface Hello implements Wo & rld { field: String }"
+        definition = assert_definitions(body, (0, 53))
+        assert isinstance(definition, InterfaceTypeDefinitionNode)
+        assert definition.name == name_node("Hello", (10, 15))
+        assert definition.description is None
+        assert definition.interfaces == [
+            type_node("Wo", (27, 29)),
+            type_node("rld", (32, 35)),
+        ]
+        assert definition.directives == []
+        assert definition.fields == [
+            field_node(
+                name_node("field", (38, 43)), type_node("String", (45, 51)), (38, 51)
+            )
+        ]
+        assert definition.loc == (0, 53)
+
     def simple_type_inheriting_multiple_interfaces_with_leading_ampersand():
         body = "type Hello implements & Wo & rld { field: String }"
         definition = assert_definitions(body, (0, 50))
@@ -357,6 +479,24 @@ def describe_schema_parser():
         ]
         assert definition.loc == (0, 50)
 
+    def simple_interface_inheriting_multiple_interfaces_with_leading_ampersand():
+        body = "interface Hello implements & Wo & rld { field: String }"
+        definition = assert_definitions(body, (0, 55))
+        assert isinstance(definition, InterfaceTypeDefinitionNode)
+        assert definition.name == name_node("Hello", (10, 15))
+        assert definition.description is None
+        assert definition.interfaces == [
+            type_node("Wo", (29, 31)),
+            type_node("rld", (34, 37)),
+        ]
+        assert definition.directives == []
+        assert definition.fields == [
+            field_node(
+                name_node("field", (40, 45)), type_node("String", (47, 53)), (40, 53)
+            )
+        ]
+        assert definition.loc == (0, 55)
+
     def single_value_enum():
         body = "enum Hello { WORLD }"
         definition = assert_definitions(body, (0, 20))
@@ -392,6 +532,7 @@ def describe_schema_parser():
         assert isinstance(definition, InterfaceTypeDefinitionNode)
         assert definition.name == name_node("Hello", (11, 16))
         assert definition.description is None
+        assert definition.interfaces == []
         assert definition.directives == []
         assert definition.fields == [
             field_node(
@@ -566,21 +707,21 @@ def describe_schema_parser():
         assert definition.loc == (0, 24)
 
     def union_fails_with_no_types():
-        assert_syntax_error("union Hello = |", "Expected Name, found <EOF>", (1, 16))
+        assert_syntax_error("union Hello = |", "Expected Name, found <EOF>.", (1, 16))
 
     def union_fails_with_leading_double_pipe():
         assert_syntax_error(
-            "union Hello = || Wo | Rld", "Expected Name, found |", (1, 16)
+            "union Hello = || Wo | Rld", "Expected Name, found '|'.", (1, 16)
         )
 
     def union_fails_with_double_pipe():
         assert_syntax_error(
-            "union Hello = Wo || Rld", "Expected Name, found |", (1, 19)
+            "union Hello = Wo || Rld", "Expected Name, found '|'.", (1, 19)
         )
 
     def union_fails_with_trailing_pipe():
         assert_syntax_error(
-            "union Hello = | Wo | Rld |", "Expected Name, found <EOF>", (1, 27)
+            "union Hello = | Wo | Rld |", "Expected Name, found <EOF>.", (1, 27)
         )
 
     def scalar():
@@ -612,7 +753,7 @@ def describe_schema_parser():
     def simple_input_object_with_args_should_fail():
         assert_syntax_error(
             "\ninput Hello {\n  world(foo : Int): String\n}",
-            "Expected :, found (",
+            "Expected ':', found '('.",
             (3, 8),
         )
 
@@ -645,7 +786,7 @@ def describe_schema_parser():
     def directive_with_incorrect_locations():
         assert_syntax_error(
             "\ndirective @foo on FIELD | INCORRECT_LOCATION",
-            "Unexpected Name 'INCORRECT_LOCATION'",
+            "Unexpected Name 'INCORRECT_LOCATION'.",
             (2, 27),
         )
 
@@ -653,13 +794,11 @@ def describe_schema_parser():
         assert parse(kitchen_sink_sdl)
 
     def disallow_legacy_sdl_empty_fields_supports_type_with_empty_fields():
-        assert_syntax_error(
-            "type Hello { }", "Syntax Error: Expected Name, found }", (1, 14)
-        )
+        assert_syntax_error("type Hello { }", "Expected Name, found '}'.", (1, 14))
 
     def disallow_legacy_sdl_implements_interfaces():
         assert_syntax_error(
             "type Hello implements Wo rld { field: String }",
-            "Syntax Error: Unexpected Name 'rld'",
+            "Unexpected Name 'rld'.",
             (1, 26),
         )
