@@ -2,7 +2,7 @@ from copy import copy
 from functools import partial
 from typing import cast, Dict, List, Optional, Tuple
 
-from pytest import raises  # type: ignore
+from pytest import mark, raises
 
 from graphql.language import (
     Node,
@@ -21,7 +21,6 @@ from graphql.language import (
 from graphql.language.visitor import QUERY_DOCUMENT_KEYS
 from graphql.pyutils import FrozenList
 
-# noinspection PyUnresolvedReferences
 from ..fixtures import kitchen_sink_query  # noqa: F401
 
 
@@ -102,17 +101,131 @@ def describe_visitor():
             visit(ast, TestVisitor())  # type: ignore
         assert str(exc_info.value) == "Not an AST Visitor: <TestVisitor instance>."
 
+    def visitors_support_all_method_variants():
+        class TestVisitorWithInstanceMethods(Visitor):
+            def enter(self, node, *args):
+                assert isinstance(self, TestVisitorWithInstanceMethods)
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter:{node.kind}")
+                pass
+
+            def leave(self, node, *args):
+                assert isinstance(self, TestVisitorWithInstanceMethods)
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave:{node.kind}")
+                pass
+
+            def enter_field(self, node, *args):
+                assert isinstance(self, TestVisitorWithInstanceMethods)
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter_field:{node.kind}")
+                pass
+
+            def leave_field(self, node, *args):
+                assert isinstance(self, TestVisitorWithInstanceMethods)
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave_field:{node.kind}")
+                pass
+
+        class TestVisitorWithClassMethods(Visitor):
+            @classmethod
+            def enter(cls, node, *args):
+                assert cls is TestVisitorWithClassMethods
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter:{node.kind}")
+                pass
+
+            @classmethod
+            def leave(cls, node, *args):
+                assert cls is TestVisitorWithClassMethods
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave:{node.kind}")
+                pass
+
+            @classmethod
+            def enter_field(cls, node, *args):
+                assert cls is TestVisitorWithClassMethods
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter_field:{node.kind}")
+                pass
+
+            @classmethod
+            def leave_field(cls, node, *args):
+                assert cls is TestVisitorWithClassMethods
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave_field:{node.kind}")
+                pass
+
+        class TestVisitorWithStaticMethods(Visitor):
+            @staticmethod
+            def enter(node, *args):
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter:{node.kind}")
+                pass
+
+            @staticmethod
+            def leave(node, *args):
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave:{node.kind}")
+                pass
+
+            @staticmethod
+            def enter_field(node, *args):
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"enter_field:{node.kind}")
+                pass
+
+            @staticmethod
+            def leave_field(node, *args):
+                assert isinstance(node, Node)
+                assert len(args) == 4
+                visited.append(f"leave_field:{node.kind}")
+                pass
+
+        for visitor_class in (
+            TestVisitorWithInstanceMethods,
+            TestVisitorWithClassMethods,
+            TestVisitorWithStaticMethods,
+        ):
+            ast = parse("{ a }")
+            visited: List[str] = []
+            visit(ast, visitor_class())
+            assert visited == [
+                "enter:document",
+                "enter:operation_definition",
+                "enter:selection_set",
+                "enter_field:field",
+                "enter:name",
+                "leave:name",
+                "leave_field:field",
+                "leave:selection_set",
+                "leave:operation_definition",
+                "leave:document",
+            ]
+
     def validates_path_argument():
         ast = parse("{ a }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 visited.append(["enter", *args[3]])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 visited.append(["leave", *args[3]])
 
@@ -134,9 +247,9 @@ def describe_visitor():
         ast = parse("{ a }", no_location=True)
         visited_nodes = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, node, key, parent, path, ancestors):
+            @staticmethod
+            def enter(node, key, parent, _path, ancestors):
                 in_array = isinstance(key, int)
                 if in_array:
                     visited_nodes.append(parent)
@@ -144,7 +257,8 @@ def describe_visitor():
                 expected_ancestors = visited_nodes[0:-2]
                 assert ancestors == expected_ancestors
 
-            def leave(self, node, key, parent, path, ancestors):
+            @staticmethod
+            def leave(_node, key, _parent, _path, ancestors):
                 expected_ancestors = visited_nodes[0:-2]
                 assert ancestors == expected_ancestors
                 in_array = isinstance(key, int)
@@ -158,14 +272,15 @@ def describe_visitor():
         ast = parse("{ a }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
             selection_set = None
 
-            def enter_field(self, node, *_args):
+            @staticmethod
+            def enter_field(node, *_args):
                 visited.append(["enter", node.kind])
 
-            def leave_field(self, node, *_args):
+            @staticmethod
+            def leave_field(node, *_args):
                 visited.append(["leave", node.kind])
 
         visit(ast, TestVisitor())
@@ -199,45 +314,46 @@ def describe_visitor():
         assert edited_ast == ast
         assert visited == ["enter", "leave"]
 
-    def allows_for_editing_on_enter():
+    @mark.parametrize("remove_action", (REMOVE, Ellipsis), ids=("REMOVE", "Ellipsis"))
+    def allows_for_editing_on_enter(remove_action):
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 if isinstance(node, FieldNode) and node.name.value == "b":
-                    return REMOVE
+                    return remove_action
 
         edited_ast = visit(ast, TestVisitor())
         assert ast == parse("{ a, b, c { a, b, c } }", no_location=True)
         assert edited_ast == parse("{ a,    c { a,    c } }", no_location=True)
 
-    def allows_for_editing_on_leave():
+    @mark.parametrize("remove_action", (REMOVE, Ellipsis), ids=("REMOVE", "Ellipsis"))
+    def allows_for_editing_on_leave(remove_action):
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args_edited(ast, *args)
                 node = args[0]
                 if isinstance(node, FieldNode) and node.name.value == "b":
-                    return REMOVE
+                    return remove_action
 
         edited_ast = visit(ast, TestVisitor())
         assert ast == parse("{ a, b, c { a, b, c } }", no_location=True)
         assert edited_ast == parse("{ a,    c { a,    c } }", no_location=True)
 
-    def ignores_false_returned_on_leave():
+    @mark.parametrize("skip_action", (SKIP, False), ids=("SKIP", "False"))
+    def ignores_false_returned_on_leave(skip_action):
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
 
-        assert SKIP is False
-
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def leave(self, *args):
-                return SKIP
+            @staticmethod
+            def leave(*args):
+                return skip_action
 
         returned_ast = visit(ast, TestVisitor())
         assert returned_ast == parse("{ a, b, c { a, b, c } }", no_location=True)
@@ -266,21 +382,23 @@ def describe_visitor():
         visit(ast, visitor)
         assert visitor.did_visit_added_field
 
-    def allows_skipping_a_sub_tree():
+    @mark.parametrize("skip_action", (SKIP, False), ids=("SKIP", "False"))
+    def allows_skipping_a_sub_tree(skip_action):
         ast = parse("{ a, b { x }, c }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
                 if kind == "field" and node.name.value == "b":
-                    return SKIP
+                    return skip_action
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -305,21 +423,23 @@ def describe_visitor():
             ["leave", "document", None],
         ]
 
-    def allows_early_exit_while_visiting():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_while_visiting(break_action):
         ast = parse("{ a, b { x }, c }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
                 if kind == "name" and node.value == "x":
-                    return BREAK
+                    return break_action
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -342,25 +462,27 @@ def describe_visitor():
             ["enter", "name", "x"],
         ]
 
-    def allows_early_exit_while_leaving():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_while_leaving(break_action):
         ast = parse("{ a, b { x }, c }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["leave", kind, value])
                 if kind == "name" and node.value == "x":
-                    return BREAK
+                    return break_action
 
         visit(ast, TestVisitor())
         assert visited == [
@@ -384,21 +506,23 @@ def describe_visitor():
         ast = parse("{ a, b { x }, c }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter_name(self, *args):
+            @staticmethod
+            def enter_name(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def enter_selection_set(self, *args):
+            @staticmethod
+            def enter_selection_set(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave_selection_set(self, *args):
+            @staticmethod
+            def leave_selection_set(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -424,15 +548,16 @@ def describe_visitor():
         )
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -479,9 +604,9 @@ def describe_visitor():
         push = arg_stack.append
         pop = arg_stack.pop
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 node, key, parent = args[:3]
                 parent_kind = parent.kind if isinstance(parent, Node) else None
                 record(["enter", node.kind, key, parent_kind])
@@ -489,7 +614,8 @@ def describe_visitor():
                 check_visitor_fn_args(ast, *args)
                 push(args[:])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 node, key, parent = args[:3]
                 parent_kind = parent.kind if isinstance(parent, Node) else None
                 record(["leave", node.kind, key, parent_kind])
@@ -868,12 +994,13 @@ def describe_support_for_custom_ast_nodes():
     def does_not_traverse_unknown_node_kinds():
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, node, *_args):
+            @staticmethod
+            def enter(node, *_args):
                 visited.append(["enter", node.kind, get_value(node)])
 
-            def leave(self, node, *_args):
+            @staticmethod
+            def leave(node, *_args):
                 visited.append(["leave", node.kind, get_value(node)])
 
         visit(custom_ast, TestVisitor())
@@ -899,12 +1026,13 @@ def describe_support_for_custom_ast_nodes():
         }
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, node, *_args):
+            @staticmethod
+            def enter(node, *_args):
                 visited.append(["enter", node.kind, get_value(node)])
 
-            def leave(self, node, *_args):
+            @staticmethod
+            def leave(node, *_args):
                 visited.append(["leave", node.kind, get_value(node)])
 
         visit(custom_ast, TestVisitor(), custom_query_document_keys)
@@ -953,22 +1081,24 @@ def describe_support_for_custom_ast_nodes():
 
 
 def describe_visit_in_parallel():
-    def allows_skipping_a_sub_tree():
+    @mark.parametrize("skip_action", (SKIP, False), ids=("SKIP", "False"))
+    def allows_skipping_a_sub_tree(skip_action):
         # Note: nearly identical to the above test but using ParallelVisitor
         ast = parse("{ a, b { x }, c }")
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
                 if kind == "field" and node.name.value == "b":
-                    return SKIP
+                    return skip_action
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -993,7 +1123,8 @@ def describe_visit_in_parallel():
             ["leave", "document", None],
         ]
 
-    def allows_skipping_different_sub_trees():
+    @mark.parametrize("skip_action", (SKIP, False), ids=("SKIP", "False"))
+    def allows_skipping_different_sub_trees(skip_action):
         ast = parse("{ a { x }, b { y} }")
         visited = []
 
@@ -1008,7 +1139,7 @@ def describe_visit_in_parallel():
                 name = self.name
                 visited.append([f"no-{name}", "enter", kind, value])
                 if kind == "field" and node.name.value == name:
-                    return SKIP
+                    return skip_action
 
             def leave(self, *args):
                 check_visitor_fn_args(ast, *args)
@@ -1055,22 +1186,24 @@ def describe_visit_in_parallel():
             ["no-b", "leave", "document", None],
         ]
 
-    def allows_early_exit_while_visiting():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_while_visiting(break_action):
         # Note: nearly identical to the above test but using ParallelVisitor.
         ast = parse("{ a, b { x }, c }")
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
                 if kind == "name" and node.value == "x":
-                    return BREAK
+                    return break_action
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -1093,7 +1226,8 @@ def describe_visit_in_parallel():
             ["enter", "name", "x"],
         ]
 
-    def allows_early_exit_from_different_points():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_from_different_points(break_action):
         ast = parse("{ a { y }, b { x } }")
         visited = []
 
@@ -1108,7 +1242,7 @@ def describe_visit_in_parallel():
                 name = self.name
                 visited.append([f"break-{name}", "enter", kind, value])
                 if kind == "name" and node.value == name:
-                    return BREAK
+                    return break_action
 
             def leave(self, *args):
                 assert self.name == "b"
@@ -1142,26 +1276,28 @@ def describe_visit_in_parallel():
             ["break-b", "enter", "name", "b"],
         ]
 
-    def allows_early_exit_while_leaving():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_while_leaving(break_action):
         # Note: nearly identical to the above test but using ParallelVisitor.
         ast = parse("{ a, b { x }, c }")
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["leave", kind, value])
                 if kind == "name" and node.value == "x":
-                    return BREAK
+                    return break_action
 
         visit(ast, ParallelVisitor([TestVisitor()]))
         assert visited == [
@@ -1181,7 +1317,8 @@ def describe_visit_in_parallel():
             ["leave", "name", "x"],
         ]
 
-    def allows_early_exit_from_leaving_different_points():
+    @mark.parametrize("break_action", (BREAK, True), ids=("BREAK", "True"))
+    def allows_early_exit_from_leaving_different_points(break_action):
         ast = parse("{ a { y }, b { x } }")
         visited = []
 
@@ -1203,7 +1340,7 @@ def describe_visit_in_parallel():
                 name = self.name
                 visited.append([f"break-{name}", "leave", kind, value])
                 if kind == "field" and node.name.value == name:
-                    return BREAK
+                    return break_action
 
         visit(ast, ParallelVisitor([TestVisitor("a"), TestVisitor("b")]))
         assert visited == [
@@ -1245,27 +1382,29 @@ def describe_visit_in_parallel():
             ["break-b", "leave", "field", None],
         ]
 
-    def allows_for_editing_on_enter():
+    @mark.parametrize("remove_action", (REMOVE, Ellipsis), ids=("REMOVE", "Ellipsis"))
+    def allows_for_editing_on_enter(remove_action):
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor1(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 if node.kind == "field" and node.name.value == "b":
-                    return REMOVE
+                    return remove_action
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor2(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args_edited(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -1301,27 +1440,29 @@ def describe_visit_in_parallel():
             ["leave", "document", None],
         ]
 
-    def allows_for_editing_on_leave():
+    @mark.parametrize("remove_action", (REMOVE, Ellipsis), ids=("REMOVE", "Ellipsis"))
+    def allows_for_editing_on_leave(remove_action):
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
         visited = []
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor1(Visitor):
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args_edited(ast, *args)
                 node = args[0]
                 if node.kind == "field" and node.name.value == "b":
-                    return REMOVE
+                    return remove_action
 
-        # noinspection PyMethodMayBeStatic
         class TestVisitor2(Visitor):
-            def enter(self, *args):
+            @staticmethod
+            def enter(*args):
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
                 visited.append(["enter", kind, value])
 
-            def leave(self, *args):
+            @staticmethod
+            def leave(*args):
                 check_visitor_fn_args_edited(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
