@@ -3,14 +3,14 @@
 from pathlib import Path
 from typing import Any, Dict, List
 
-from graphql.pyutils import dedent
+from .utils import dedent
 
 Scope = Dict[str, Any]
 
 
 def get_snippets(source, indent=4):
     """Get all code snippets from a given documentation source file."""
-    if not source.endswith(".rst"):
+    if not source.endswith(".rst"):  # pragma: no cover
         source += ".rst"
     source_path = Path(__file__).parents[1] / "docs" / source
     lines = open(source_path).readlines()
@@ -134,17 +134,11 @@ def describe_usage():
 
         async_query = queries.pop(0)
         assert "asyncio" in async_query and "graphql_sync" not in async_query
-        assert "asyncio.run" in async_query
-        try:  # pragma: no cover
-            from asyncio import run  # noqa: F401
-        except ImportError:  # Python < 3.7
-            assert "ExecutionResult" in expected_result(queries)
-        else:  # pragma: no cover
-            exec(async_query, scope)
-            out, err = capsys.readouterr()
-            assert not err
-            assert "R2-D2" in out
-            assert out == expected_result(queries)
+        exec(async_query, scope)
+        out, err = capsys.readouterr()
+        assert not err
+        assert "R2-D2" in out
+        assert out == expected_result(queries)
 
         sync_query = queries.pop(0)
         assert "graphql_sync" in sync_query and "asyncio" not in sync_query
@@ -155,7 +149,7 @@ def describe_usage():
         assert out == expected_result(queries)
 
         bad_query = queries.pop(0)
-        assert "homeTown" in bad_query
+        assert "homePlace" in bad_query
         exec(bad_query, scope)
         out, err = capsys.readouterr()
         assert not err
