@@ -1,7 +1,7 @@
 from math import isnan, nan
 from typing import cast
 
-from pytest import mark, raises  # type: ignore
+from pytest import mark, raises
 
 from graphql.error import GraphQLError
 from graphql.language import (
@@ -22,6 +22,7 @@ from graphql.language import (
     StringValueNode,
     TypeDefinitionNode,
     TypeExtensionNode,
+    ValueNode,
     UnionTypeDefinitionNode,
     UnionTypeExtensionNode,
 )
@@ -68,6 +69,7 @@ def describe_type_system_scalars():
         assert kwargs == {
             "name": "SomeScalar",
             "description": None,
+            "specified_by_url": None,
             "serialize": None,
             "parse_value": None,
             "parse_literal": None,
@@ -90,6 +92,12 @@ def describe_type_system_scalars():
         assert scalar.description is description
         assert scalar.to_kwargs()["description"] is description
 
+    def accepts_a_scalar_type_defining_specified_by_url():
+        url = "https://example.com/foo_spec"
+        scalar = GraphQLScalarType("SomeScalar", specified_by_url=url)
+        assert scalar.specified_by_url == url
+        assert scalar.to_kwargs()["specified_by_url"] == url
+
     def accepts_a_scalar_type_defining_parse_value_and_parse_literal():
         def parse_value(_value):
             pass
@@ -132,6 +140,10 @@ def describe_type_system_scalars():
             scalar.parse_literal(parse_value('{foo: "bar"}'))
             == "parse_value: {'foo': 'bar'}"
         )
+        assert (
+            scalar.parse_literal(parse_value("{foo: { bar: $var } }"), {"var": "baz"})
+            == "parse_value: {'foo': {'bar': 'baz'}}"
+        )
 
     def accepts_a_scalar_type_with_ast_node_and_extension_ast_nodes():
         ast_node = ScalarTypeDefinitionNode()
@@ -173,6 +185,15 @@ def describe_type_system_scalars():
             GraphQLScalarType("SomeScalar", description=[])  # type: ignore
         assert str(exc_info.value) == "The description must be a string."
 
+    def rejects_a_scalar_type_defining_specified_by_url_with_an_incorrect_type():
+        with raises(TypeError) as exc_info:
+            # noinspection PyTypeChecker
+            GraphQLScalarType("SomeScalar", specified_by_url={})  # type: ignore
+        assert (
+            str(exc_info.value)
+            == "SomeScalar must provide 'specified_by_url' as a string, but got: {}."
+        )
+
     def rejects_a_scalar_type_defining_serialize_with_incorrect_type():
         with raises(TypeError) as exc_info:
             # noinspection PyTypeChecker
@@ -185,10 +206,11 @@ def describe_type_system_scalars():
         )
 
     def rejects_a_scalar_type_defining_parse_literal_but_not_parse_value():
+        def parse_literal(_node: ValueNode, _vars=None):
+            return Undefined  # pragma: no cover
+
         with raises(TypeError) as exc_info:
-            GraphQLScalarType(
-                "SomeScalar", parse_literal=lambda: None  # pragma: no cover
-            )
+            GraphQLScalarType("SomeScalar", parse_literal=parse_literal)
         assert str(exc_info.value) == (
             "SomeScalar must provide both"
             " 'parse_value' and 'parse_literal' as functions."
@@ -435,6 +457,7 @@ def describe_type_system_objects():
         assert arg.description is None
         assert arg.type is ScalarType
         assert arg.default_value is Undefined
+        assert arg.deprecation_reason is None
         assert arg.extensions is None
         assert arg.ast_node is None
         assert field.resolve is None
@@ -939,7 +962,7 @@ def describe_type_system_unions():
             GraphQLUnionType()  # type: ignore
         with raises(TypeError) as exc_info:
             # noinspection PyTypeChecker
-            GraphQLUnionType(None, [])
+            GraphQLUnionType(None, [])  # type: ignore
         assert str(exc_info.value) == "Must provide name."
         with raises(TypeError) as exc_info:
             GraphQLUnionType("", [])
@@ -1446,6 +1469,7 @@ def describe_type_system_input_objects():
             assert input_field.description is None
             assert input_field.type is ScalarType
             assert input_field.default_value is Undefined
+            assert input_field.deprecation_reason is None
             assert input_field.extensions is None
             assert input_field.ast_node is None
             assert input_field.out_name is None
@@ -1469,6 +1493,7 @@ def describe_type_system_input_objects():
             assert input_field.description is None
             assert input_field.type is ScalarType
             assert input_field.default_value is Undefined
+            assert input_field.deprecation_reason is None
             assert input_field.extensions is None
             assert input_field.ast_node is None
             assert input_field.out_name is None
@@ -1534,17 +1559,14 @@ def describe_type_system_input_objects():
                     "SomeInputObject",
                     {
                         "f": GraphQLInputField(  # type: ignore
-                            ScalarType, resolve=resolve,
+                            ScalarType,
+                            resolve=resolve,
                         )
                     },
                 )
             input_obj_type = GraphQLInputObjectType(
                 "SomeInputObject",
-                {
-                    "f": GraphQLField(  # type: ignore
-                        ScalarType, resolve=resolve
-                    )
-                },
+                {"f": GraphQLField(ScalarType, resolve=resolve)},  # type: ignore
             )
             with raises(TypeError) as exc_info:
                 if input_obj_type.fields:
@@ -1609,6 +1631,12 @@ def describe_type_system_arguments():
             GraphQLArgument(GraphQLString, description=[])  # type: ignore
         assert str(exc_info.value) == "Argument description must be a string."
 
+    def rejects_an_argument_with_an_incorrectly_typed_deprecation_reason():
+        with raises(TypeError) as exc_info:
+            # noinspection PyTypeChecker
+            GraphQLArgument(GraphQLString, deprecation_reason=[])  # type: ignore
+        assert str(exc_info.value) == "Argument deprecation reason must be a string."
+
     def rejects_an_argument_with_an_incorrect_out_name():
         # This is an extension of GraphQL.js.
         with raises(TypeError) as exc_info:
@@ -1668,6 +1696,12 @@ def describe_type_system_input_fields():
             GraphQLInputField(GraphQLString, description=[])  # type: ignore
         assert str(exc_info.value) == "Input field description must be a string."
 
+    def rejects_an_input_field_with_an_incorrectly_typed_deprecation_reason():
+        with raises(TypeError) as exc_info:
+            # noinspection PyTypeChecker
+            GraphQLInputField(GraphQLString, deprecation_reason=[])  # type: ignore
+        assert str(exc_info.value) == "Input field deprecation reason must be a string."
+
     def rejects_an_input_field_with_an_incorrect_out_name():
         # This is an extension of GraphQL.js.
         with raises(TypeError) as exc_info:
