@@ -1,6 +1,7 @@
 from math import inf, nan, pi
+from typing import Any
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql.error import GraphQLError
 from graphql.language import parse_value as parse_value_to_ast
@@ -19,7 +20,7 @@ def describe_type_system_specified_scalar_types():
         def parse_value():
             _parse_value = GraphQLInt.parse_value
 
-            def _parse_value_raises(s, message):
+            def _parse_value_raises(s: Any, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_value(s)
                 assert str(exc_info.value) == message
@@ -53,10 +54,10 @@ def describe_type_system_specified_scalar_types():
             )
 
         def parse_literal():
-            def _parse_literal(s):
+            def _parse_literal(s: str):
                 return GraphQLInt.parse_literal(parse_value_to_ast(s))
 
-            def _parse_literal_raises(s, message):
+            def _parse_literal_raises(s: str, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_literal(s)
                 assert str(exc_info.value).startswith(message + "\n")
@@ -101,11 +102,81 @@ def describe_type_system_specified_scalar_types():
                 "$var", "Int cannot represent non-integer value: $var"
             )
 
+        def serializes():
+            serialize = GraphQLInt.serialize
+
+            assert serialize(1) == 1
+            assert serialize("123") == 123
+            assert serialize(0) == 0
+            assert serialize(-1) == -1
+            assert serialize(1e5) == 100000
+            assert serialize(False) == 0
+            assert serialize(True) == 1
+            assert serialize(type("Int", (int,), {})(5)) == 5
+
+            # The GraphQL specification does not allow serializing non-integer
+            # values as Int to avoid accidental data loss.
+            with raises(GraphQLError) as exc_info:
+                serialize(0.1)
+            assert str(exc_info.value) == "Int cannot represent non-integer value: 0.1"
+            with raises(GraphQLError) as exc_info:
+                serialize(1.1)
+            assert str(exc_info.value) == "Int cannot represent non-integer value: 1.1"
+            with raises(GraphQLError) as exc_info:
+                serialize(-1.1)
+            assert str(exc_info.value) == "Int cannot represent non-integer value: -1.1"
+            with raises(GraphQLError) as exc_info:
+                serialize("-1.1")
+            assert (
+                str(exc_info.value) == "Int cannot represent non-integer value: '-1.1'"
+            )
+            # Maybe a safe JavaScript int, but bigger than 2^32, so not
+            # representable as a GraphQL Int
+            with raises(GraphQLError) as exc_info:
+                serialize(9876504321)
+            assert str(exc_info.value) == (
+                "Int cannot represent non 32-bit signed integer value: 9876504321"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize(-9876504321)
+            assert str(exc_info.value) == (
+                "Int cannot represent non 32-bit signed integer value: -9876504321"
+            )
+            # Too big to represent as an Int in JavaScript or GraphQL
+            with raises(GraphQLError) as exc_info:
+                serialize(1e100)
+            assert str(exc_info.value) == (
+                "Int cannot represent non 32-bit signed integer value: 1e+100"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize(-1e100)
+            assert str(exc_info.value) == (
+                "Int cannot represent non 32-bit signed integer value: -1e+100"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize("one")
+            assert (
+                str(exc_info.value) == "Int cannot represent non-integer value: 'one'"
+            )
+            # Doesn't represent number
+            with raises(GraphQLError) as exc_info:
+                serialize("")
+            assert str(exc_info.value) == "Int cannot represent non-integer value: ''"
+            with raises(GraphQLError) as exc_info:
+                serialize(nan)
+            assert str(exc_info.value) == "Int cannot represent non-integer value: nan"
+            with raises(GraphQLError) as exc_info:
+                serialize(inf)
+            assert str(exc_info.value) == "Int cannot represent non-integer value: inf"
+            with raises(GraphQLError) as exc_info:
+                serialize([5])
+            assert str(exc_info.value) == "Int cannot represent non-integer value: [5]"
+
     def describe_graphql_float():
         def parse_value():
             _parse_value = GraphQLFloat.parse_value
 
-            def _parse_value_raises(s, message):
+            def _parse_value_raises(s: Any, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_value(s)
                 assert str(exc_info.value) == message
@@ -138,10 +209,10 @@ def describe_type_system_specified_scalar_types():
             )
 
         def parse_literal():
-            def _parse_literal(s):
+            def _parse_literal(s: str):
                 return GraphQLFloat.parse_literal(parse_value_to_ast(s))
 
-            def _parse_literal_raises(s, message):
+            def _parse_literal_raises(s: str, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_literal(s)
                 assert str(exc_info.value).startswith(message + "\n")
@@ -184,11 +255,50 @@ def describe_type_system_specified_scalar_types():
                 "$var", "Float cannot represent non numeric value: $var"
             )
 
+        def serializes():
+            serialize = GraphQLFloat.serialize
+
+            assert serialize(1) == 1.0
+            assert serialize(0) == 0.0
+            assert serialize("123.5") == 123.5
+            assert serialize(-1) == -1.0
+            assert serialize(0.1) == 0.1
+            assert serialize(1.1) == 1.1
+            assert serialize(-1.1) == -1.1
+            assert serialize("-1.1") == -1.1
+            assert serialize(False) == 0
+            assert serialize(True) == 1
+            assert serialize(type("Float", (float,), {})(5.5)) == 5.5
+
+            with raises(GraphQLError) as exc_info:
+                serialize(nan)
+            assert (
+                str(exc_info.value) == "Float cannot represent non numeric value: nan"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize(inf)
+            assert (
+                str(exc_info.value) == "Float cannot represent non numeric value: inf"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize("one")
+            assert str(exc_info.value) == (
+                "Float cannot represent non numeric value: 'one'"
+            )
+            with raises(GraphQLError) as exc_info:
+                serialize("")
+            assert str(exc_info.value) == "Float cannot represent non numeric value: ''"
+            with raises(GraphQLError) as exc_info:
+                serialize([5])
+            assert (
+                str(exc_info.value) == "Float cannot represent non numeric value: [5]"
+            )
+
     def describe_graphql_string():
         def parse_value():
             _parse_value = GraphQLString.parse_value
 
-            def _parse_value_raises(s, message):
+            def _parse_value_raises(s: Any, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_value(s)
                 assert str(exc_info.value) == message
@@ -215,10 +325,10 @@ def describe_type_system_specified_scalar_types():
             )
 
         def parse_literal():
-            def _parse_literal(s):
+            def _parse_literal(s: str):
                 return GraphQLString.parse_literal(parse_value_to_ast(s))
 
-            def _parse_literal_raises(s, message):
+            def _parse_literal_raises(s: str, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_literal(s)
                 assert str(exc_info.value).startswith(message + "\n")
@@ -256,11 +366,45 @@ def describe_type_system_specified_scalar_types():
                 "$var", "String cannot represent a non string value: $var"
             )
 
+        def serializes():
+            serialize = GraphQLString.serialize
+
+            assert serialize("string") == "string"
+            assert serialize(1) == "1"
+            assert serialize(-1.1) == "-1.1"
+            assert serialize(True) == "true"
+            assert serialize(False) == "false"
+
+            class StringableObjValue:
+                def __str__(self):
+                    return "something useful"
+
+            assert serialize(StringableObjValue()) == "something useful"
+
+            with raises(GraphQLError) as exc_info:
+                serialize(nan)
+            assert str(exc_info.value) == "String cannot represent value: nan"
+
+            with raises(GraphQLError) as exc_info:
+                serialize([1])
+            assert str(exc_info.value) == "String cannot represent value: [1]"
+
+            with raises(GraphQLError) as exc_info:
+                serialize({})
+            assert str(exc_info.value) == "String cannot represent value: {}"
+
+            with raises(GraphQLError) as exc_info:
+                serialize({"value_of": "value_of string"})
+            assert (
+                str(exc_info.value) == "String cannot represent value:"
+                " {'value_of': 'value_of string'}"
+            )
+
     def describe_graphql_boolean():
         def parse_value():
             _parse_value = GraphQLBoolean.parse_value
 
-            def _parse_value_raises(s, message):
+            def _parse_value_raises(s: Any, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_value(s)
                 assert str(exc_info.value) == message
@@ -295,10 +439,10 @@ def describe_type_system_specified_scalar_types():
             )
 
         def parse_literal():
-            def _parse_literal(s):
+            def _parse_literal(s: str):
                 return GraphQLBoolean.parse_literal(parse_value_to_ast(s))
 
-            def _parse_literal_raises(s, message):
+            def _parse_literal_raises(s: str, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_literal(s)
                 assert str(exc_info.value).startswith(message + "\n")
@@ -357,11 +501,52 @@ def describe_type_system_specified_scalar_types():
                 "$var", "Boolean cannot represent a non boolean value: $var"
             )
 
+        def serializes():
+            serialize = GraphQLBoolean.serialize
+
+            assert serialize(1) is True
+            assert serialize(0) is False
+            assert serialize(True) is True
+            assert serialize(False) is False
+            with raises(TypeError, match="not an acceptable base type"):
+                # you can't subclass bool in Python
+                assert serialize(type("Boolean", (bool,), {})(True)) is True
+
+            with raises(GraphQLError) as exc_info:
+                serialize(nan)
+            assert str(exc_info.value) == (
+                "Boolean cannot represent a non boolean value: nan"
+            )
+
+            with raises(GraphQLError) as exc_info:
+                serialize("")
+            assert str(exc_info.value) == (
+                "Boolean cannot represent a non boolean value: ''"
+            )
+
+            with raises(GraphQLError) as exc_info:
+                serialize("True")
+            assert str(exc_info.value) == (
+                "Boolean cannot represent a non boolean value: 'True'"
+            )
+
+            with raises(GraphQLError) as exc_info:
+                serialize([False])
+            assert str(exc_info.value) == (
+                "Boolean cannot represent a non boolean value: [False]"
+            )
+
+            with raises(GraphQLError) as exc_info:
+                serialize({})
+            assert str(exc_info.value) == (
+                "Boolean cannot represent a non boolean value: {}"
+            )
+
     def describe_graphql_id():
         def parse_value():
             _parse_value = GraphQLID.parse_value
 
-            def _parse_value_raises(s, message):
+            def _parse_value_raises(s: Any, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_value(s)
                 assert str(exc_info.value) == message
@@ -389,10 +574,10 @@ def describe_type_system_specified_scalar_types():
             )
 
         def parse_literal():
-            def _parse_literal(s):
+            def _parse_literal(s: str):
                 return GraphQLID.parse_literal(parse_value_to_ast(s))
 
-            def _parse_literal_raises(s, message):
+            def _parse_literal_raises(s: str, message: str):
                 with raises(GraphQLError) as exc_info:
                     _parse_literal(s)
                 assert str(exc_info.value).startswith(message + "\n")
@@ -439,3 +624,39 @@ def describe_type_system_specified_scalar_types():
             _parse_literal_raises(
                 "$var", "ID cannot represent a non-string and non-integer value: $var"
             )
+
+        def serializes():
+            serialize = GraphQLID.serialize
+
+            assert serialize("string") == "string"
+            assert serialize("false") == "false"
+            assert serialize("") == ""
+            assert serialize(123) == "123"
+            assert serialize(0) == "0"
+            assert serialize(-1) == "-1"
+
+            class ObjValue:
+                def __init__(self, value):
+                    self._id = value
+
+                def __str__(self):
+                    return str(self._id)
+
+            obj_value = ObjValue(123)
+            assert serialize(obj_value) == "123"
+
+            with raises(GraphQLError) as exc_info:
+                serialize(True)
+            assert str(exc_info.value) == "ID cannot represent value: True"
+
+            with raises(GraphQLError) as exc_info:
+                serialize(3.14)
+            assert str(exc_info.value) == "ID cannot represent value: 3.14"
+
+            with raises(GraphQLError) as exc_info:
+                serialize({})
+            assert str(exc_info.value) == "ID cannot represent value: {}"
+
+            with raises(GraphQLError) as exc_info:
+                serialize(["abc"])
+            assert str(exc_info.value) == "ID cannot represent value: ['abc']"
