@@ -1,4 +1,6 @@
-from pytest import raises  # type: ignore
+from copy import deepcopy
+
+from pytest import raises
 
 from graphql.language import (
     DirectiveLocation,
@@ -7,7 +9,7 @@ from graphql.language import (
     TypeDefinitionNode,
     TypeExtensionNode,
 )
-from graphql.pyutils import dedent, FrozenList
+from graphql.pyutils import FrozenList
 from graphql.type import (
     GraphQLArgument,
     GraphQLBoolean,
@@ -26,7 +28,9 @@ from graphql.type import (
     GraphQLType,
     specified_directives,
 )
-from graphql.utilities import print_schema
+from graphql.utilities import build_schema, lexicographic_sort_schema, print_schema
+
+from ..utils import dedent
 
 
 def describe_type_system_schema():
@@ -93,7 +97,10 @@ def describe_type_system_schema():
         )
 
         schema = GraphQLSchema(
-            BlogQuery, BlogMutation, BlogSubscription, description="Sample schema",
+            BlogQuery,
+            BlogMutation,
+            BlogSubscription,
+            description="Sample schema",
         )
 
         kwargs = schema.to_kwargs()
@@ -197,7 +204,9 @@ def describe_type_system_schema():
             SomeSubtype = GraphQLObjectType(
                 "SomeSubtype", {}, interfaces=lambda: [SomeInterface]
             )
-            schema = GraphQLSchema(types=[SomeSubtype],)
+            schema = GraphQLSchema(
+                types=[SomeSubtype],
+            )
             assert schema.type_map["SomeInterface"] is SomeInterface
             assert schema.type_map["AnotherInterface"] is AnotherInterface
             assert schema.type_map["SomeSubtype"] is SomeSubtype
@@ -432,3 +441,52 @@ def describe_type_system_schema():
                 "Schema extension AST nodes must be specified"
                 " as a collection of SchemaExtensionNode instances."
             )
+
+    def can_deep_copy_a_schema():
+        source = """
+            schema {
+              query: Farm
+              mutation: Work
+            }
+
+            type Cow {
+              id: ID!
+              name: String
+              moos: Boolean
+            }
+
+            type Pig {
+              id: ID!
+              name: String
+              oink: Boolean
+            }
+
+            union Animal = Cow | Pig
+
+            enum Food {
+              CORN
+              FRUIT
+            }
+
+            input Feed {
+              amount: Float
+              type: Food
+            }
+
+            type Farm {
+              animals: [Animal!]!
+            }
+
+            type Work {
+              feed(feed: Feed): Boolean
+            }
+        """
+        schema = build_schema(source)
+        schema_copy = deepcopy(schema)
+
+        for name in ("Cow", "Pig", "Animal", "Food", "Feed", "Farm", "Work"):
+            assert schema.get_type(name) is not schema_copy.get_type(name)
+
+        assert print_schema(lexicographic_sort_schema(schema)) == print_schema(
+            lexicographic_sort_schema(schema_copy)
+        )
