@@ -1,8 +1,8 @@
-from math import nan
+from math import inf, nan
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
-from graphql.error import INVALID
+from graphql.error import GraphQLError
 from graphql.language import (
     BooleanValueNode,
     EnumValueNode,
@@ -15,6 +15,7 @@ from graphql.language import (
     ObjectValueNode,
     StringValueNode,
 )
+from graphql.pyutils import Undefined
 from graphql.type import (
     GraphQLBoolean,
     GraphQLEnumType,
@@ -37,9 +38,7 @@ def describe_ast_from_value():
 
         assert ast_from_value(False, GraphQLBoolean) == BooleanValueNode(value=False)
 
-        assert ast_from_value(INVALID, GraphQLBoolean) is None
-
-        assert ast_from_value(nan, GraphQLInt) is None
+        assert ast_from_value(Undefined, GraphQLBoolean) is None
 
         assert ast_from_value(None, GraphQLBoolean) == NullValueNode()
 
@@ -59,17 +58,22 @@ def describe_ast_from_value():
 
         # GraphQL spec does not allow coercing non-integer values to Int to
         # avoid accidental data loss.
-        with raises(TypeError) as exc_info:
+        with raises(GraphQLError) as exc_info:
             assert ast_from_value(123.5, GraphQLInt)
         msg = str(exc_info.value)
         assert msg == "Int cannot represent non-integer value: 123.5"
 
         # Note: outside the bounds of 32bit signed int.
-        with raises(TypeError) as exc_info:
+        with raises(GraphQLError) as exc_info:
             assert ast_from_value(1e40, GraphQLInt)
         msg = str(exc_info.value)
         assert msg == "Int cannot represent non 32-bit signed integer value: 1e+40"
 
+        with raises(GraphQLError) as exc_info:
+            ast_from_value(nan, GraphQLInt)
+        msg = str(exc_info.value)
+        assert msg == "Int cannot represent non-integer value: nan"
+
     def converts_float_values_to_float_asts():
         # luckily in Python we can discern between float and int
         assert ast_from_value(-1, GraphQLFloat) == FloatValueNode(value="-1")
@@ -97,7 +101,7 @@ def describe_ast_from_value():
 
         assert ast_from_value(None, GraphQLString) == NullValueNode()
 
-        assert ast_from_value(INVALID, GraphQLString) is None
+        assert ast_from_value(Undefined, GraphQLString) is None
 
     def converts_id_values_to_int_or_string_asts():
         assert ast_from_value("hello", GraphQLID) == StringValueNode(value="hello")
@@ -116,13 +120,13 @@ def describe_ast_from_value():
 
         assert ast_from_value("01", GraphQLID) == StringValueNode(value="01")
 
-        with raises(TypeError) as exc_info:
+        with raises(GraphQLError) as exc_info:
             assert ast_from_value(False, GraphQLID)
         assert str(exc_info.value) == "ID cannot represent value: False"
 
         assert ast_from_value(None, GraphQLID) == NullValueNode()
 
-        assert ast_from_value(INVALID, GraphQLString) is None
+        assert ast_from_value(Undefined, GraphQLString) is None
 
     def converts_using_serialize_from_a_custom_scalar_type():
         pass_through_scalar = GraphQLScalarType(
@@ -134,6 +138,14 @@ def describe_ast_from_value():
             value="value"
         )
 
+        with raises(TypeError) as exc_info:
+            assert ast_from_value(nan, pass_through_scalar)
+        assert str(exc_info.value) == "Cannot convert value to AST: nan."
+
+        with raises(TypeError) as exc_info:
+            ast_from_value(inf, pass_through_scalar)
+        assert str(exc_info.value) == "Cannot convert value to AST: inf."
+
         return_null_scalar = GraphQLScalarType(
             "ReturnNullScalar",
             serialize=lambda value: None,
@@ -152,7 +164,7 @@ def describe_ast_from_value():
         with raises(TypeError) as exc_info:
             ast_from_value("value", return_custom_class_scalar)
         msg = str(exc_info.value)
-        assert msg == "Cannot convert value to AST: <SomeClass instance>"
+        assert msg == "Cannot convert value to AST: <SomeClass instance>."
 
     def does_not_convert_non_null_values_to_null_value():
         non_null_boolean = GraphQLNonNull(GraphQLBoolean)
@@ -170,10 +182,17 @@ def describe_ast_from_value():
         assert ast_from_value(complex_value, my_enum) == EnumValueNode(value="COMPLEX")
 
         # Note: case sensitive
-        assert ast_from_value("hello", my_enum) is None
+        with raises(GraphQLError) as exc_info:
+            ast_from_value("hello", my_enum)
+        assert exc_info.value.message == "Enum 'MyEnum' cannot represent value: 'hello'"
 
         # Note: not a valid enum value
-        assert ast_from_value("VALUE", my_enum) is None
+        with raises(GraphQLError) as exc_info:
+            ast_from_value("UNKNOWN_VALUE", my_enum)
+        assert (
+            exc_info.value.message
+            == "Enum 'MyEnum' cannot represent value: 'UNKNOWN_VALUE'"
+        )
 
     def converts_list_values_to_list_asts():
         assert ast_from_value(
@@ -223,3 +242,6 @@ def describe_ast_from_value():
         assert ast_from_value({"foo": None}, input_obj) == ObjectValueNode(
             fields=[ObjectFieldNode(name=NameNode(value="foo"), value=NullValueNode())]
         )
+
+    def does_not_convert_non_object_values_as_input_objects():
+        assert ast_from_value(5, input_obj) is None
