@@ -1,20 +1,23 @@
 from collections import namedtuple
 from typing import Union
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql import graphql_sync
-from graphql.language import parse, print_ast, DocumentNode
+from graphql.language import parse, print_ast, DocumentNode, InterfaceTypeDefinitionNode
 from graphql.type import (
     GraphQLDeprecatedDirective,
     GraphQLIncludeDirective,
+    GraphQLSchema,
     GraphQLSkipDirective,
+    GraphQLSpecifiedByDirective,
     GraphQLBoolean,
     GraphQLFloat,
     GraphQLID,
     GraphQLInt,
     GraphQLString,
     GraphQLArgument,
+    GraphQLEnumType,
     GraphQLEnumValue,
     GraphQLField,
     GraphQLInputField,
@@ -26,10 +29,12 @@ from graphql.type import (
     assert_object_type,
     assert_scalar_type,
     assert_union_type,
+    introspection_types,
     validate_schema,
 )
-from graphql.pyutils import dedent
-from graphql.utilities import build_ast_schema, build_schema, print_schema
+from graphql.utilities import build_ast_schema, build_schema, print_schema, print_type
+
+from ..utils import dedent
 
 
 def cycle_sdl(sdl: str) -> str:
@@ -48,12 +53,19 @@ TypeWithAstNode = Union[
     GraphQLArgument, GraphQLEnumValue, GraphQLField, GraphQLInputField, GraphQLNamedType
 ]
 
+TypeWithExtensionAstNodes = GraphQLNamedType
+
 
 def print_ast_node(obj: TypeWithAstNode) -> str:
     assert obj is not None and obj.ast_node is not None
     return print_ast(obj.ast_node)
 
 
+def print_all_ast_nodes(obj: TypeWithExtensionAstNodes) -> str:
+    assert obj is not None and obj.extension_ast_nodes is not None
+    return print_ast(DocumentNode(definitions=[obj.ast_node, *obj.extension_ast_nodes]))
+
+
 def describe_schema_builder():
     def can_use_built_schema_for_limited_execution():
         schema = build_ast_schema(
@@ -66,9 +78,9 @@ def describe_schema_builder():
             )
         )
 
-        data = namedtuple("Data", "str")(123)  # type: ignore
+        root_value = namedtuple("Data", "str")(123)  # type: ignore
 
-        result = graphql_sync(schema, "{ str }", data)
+        result = graphql_sync(schema=schema, source="{ str }", root_value=root_value)
         assert result == ({"str": "123"}, None)
 
     def can_build_a_schema_directly_from_the_source():
@@ -79,17 +91,37 @@ def describe_schema_builder():
             }
             """
         )
+        source = "{ add(x: 34, y: 55) }"
 
         # noinspection PyMethodMayBeStatic
-        class Root:
+        class RootValue:
             def add(self, _info, x, y):
                 return x + y
 
-        assert graphql_sync(schema, "{ add(x: 34, y: 55) }", Root()) == (
+        assert graphql_sync(schema=schema, source=source, root_value=RootValue()) == (
             {"add": 89},
             None,
         )
 
+    def ignores_non_type_system_definitions():
+        sdl = """
+            type Query {
+              str: String
+            }
+
+            fragment SomeFragment on Query {
+              str
+            }
+            """
+        build_schema(sdl)
+
+    def match_order_of_default_types_and_directives():
+        schema = GraphQLSchema()
+        sdl_schema = build_ast_schema(DocumentNode(definitions=[]))
+
+        assert sdl_schema.directives == schema.directives
+        assert sdl_schema.type_map == schema.type_map
+
     def empty_type():
         sdl = dedent(
             """
@@ -143,12 +175,35 @@ def describe_schema_builder():
     def supports_descriptions():
         sdl = dedent(
             '''
+            """Do you agree that this is the most creative schema ever?"""
+            schema {
+              query: Query
+            }
+
             """This is a directive"""
             directive @foo(
               """It has an argument"""
               arg: Int
             ) on FIELD
 
+            """Who knows what's inside this scalar?"""
+            scalar MysteryScalar
+
+            """This is an input object type"""
+            input FooInput {
+              """It has a field"""
+              field: Int
+            }
+
+            """This is an interface type"""
+            interface Energy {
+              """It also has a field"""
+              str: String
+            }
+
+            """There is nothing inside!"""
+            union BlackHole
+
             """With an enum"""
             enum Color {
               RED
@@ -167,13 +222,14 @@ def describe_schema_builder():
         )
         assert cycle_sdl(sdl) == sdl
 
-    def maintains_skip_and_include_directives():
+    def maintains_include_skip_and_specified_by_url_directives():
         schema = build_schema("type Query")
 
-        assert len(schema.directives) == 3
+        assert len(schema.directives) == 4
         assert schema.get_directive("skip") is GraphQLSkipDirective
         assert schema.get_directive("include") is GraphQLIncludeDirective
         assert schema.get_directive("deprecated") is GraphQLDeprecatedDirective
+        assert schema.get_directive("specifiedBy") is GraphQLSpecifiedByDirective
 
     def overriding_directives_excludes_specified():
         schema = build_schema(
@@ -181,10 +237,11 @@ def describe_schema_builder():
             directive @skip on FIELD
             directive @include on FIELD
             directive @deprecated on FIELD_DEFINITION
+            directive @specifiedBy on FIELD_DEFINITION
             """
         )
 
-        assert len(schema.directives) == 3
+        assert len(schema.directives) == 4
         get_directive = schema.get_directive
         assert get_directive("skip") is not GraphQLSkipDirective
         assert get_directive("skip") is not None
@@ -192,18 +249,21 @@ def describe_schema_builder():
         assert get_directive("include") is not None
         assert get_directive("deprecated") is not GraphQLDeprecatedDirective
         assert get_directive("deprecated") is not None
+        assert get_directive("specifiedBy") is not GraphQLSpecifiedByDirective
+        assert get_directive("specifiedBy") is not None
 
-    def adding_directives_maintains_skip_and_include_directives():
+    def adding_directives_maintains_include_skip_and_specified_by_directives():
         schema = build_schema(
             """
             directive @foo(arg: Int) on FIELD
             """
         )
 
-        assert len(schema.directives) == 4
+        assert len(schema.directives) == 5
         assert schema.get_directive("skip") is GraphQLSkipDirective
         assert schema.get_directive("include") is GraphQLIncludeDirective
         assert schema.get_directive("deprecated") is GraphQLDeprecatedDirective
+        assert schema.get_directive("specifiedBy") is GraphQLSpecifiedByDirective
         assert schema.get_directive("foo") is not None
 
     def type_modifiers():
@@ -211,10 +271,10 @@ def describe_schema_builder():
             """
             type Query {
               nonNullStr: String!
-              listOfStrs: [String]
-              listOfNonNullStrs: [String!]
-              nonNullListOfStrs: [String]!
-              nonNullListOfNonNullStrs: [String!]!
+              listOfStrings: [String]
+              listOfNonNullStrings: [String!]
+              nonNullListOfStrings: [String]!
+              nonNullListOfNonNullStrings: [String!]!
             }
             """
         )
@@ -277,6 +337,11 @@ def describe_schema_builder():
             interface EmptyInterface
             """
         )
+
+        definition = parse(sdl).definitions[0]
+        assert isinstance(definition, InterfaceTypeDefinitionNode)
+        assert definition.interfaces == []
+
         assert cycle_sdl(sdl) == sdl
 
     def simple_type_with_interface():
@@ -293,6 +358,28 @@ def describe_schema_builder():
         )
         assert cycle_sdl(sdl) == sdl
 
+    def simple_interface_hierarchy():
+        sdl = dedent(
+            """
+            schema {
+              query: Child
+            }
+
+            interface Child implements Parent {
+              str: String
+            }
+
+            type Hello implements Parent & Child {
+              str: String
+            }
+
+            interface Parent {
+              str: String
+            }
+            """
+        )
+        assert cycle_sdl(sdl) == sdl
+
     def empty_enum():
         sdl = dedent(
             """
@@ -344,6 +431,12 @@ def describe_schema_builder():
         )
         assert cycle_sdl(sdl) == sdl
 
+        # check that the internal values are the same as the names
+        schema = build_schema(sdl)
+        enum_type = schema.get_type("Hello")
+        assert isinstance(enum_type, GraphQLEnumType)
+        assert [value.value for value in enum_type.values.values()] == ["WO", "RLD"]
+
     def empty_union():
         sdl = dedent(
             """
@@ -400,148 +493,11 @@ def describe_schema_builder():
                 }
                 """
             )
-        msg = str(exc_info.value)
         assert (
-            msg == "Hello types must be specified"
-            " as a sequence of GraphQLObjectType instances."
+            str(exc_info.value) == "Hello types must be specified"
+            " as a collection of GraphQLObjectType instances."
         )
 
-    def describe_specifying_union_type_using_typename():
-
-        schema = build_schema(
-            """
-            type Query {
-              fruits: [Fruit]
-            }
-
-            union Fruit = Apple | Banana
-
-            type Apple {
-              color: String
-            }
-
-            type Banana {
-              length: Int
-            }
-            """
-        )
-
-        query = """
-            {
-              fruits {
-                ... on Apple {
-                  color
-                }
-                ... on Banana {
-                  length
-                }
-              }
-            }
-            """
-
-        expected = ({"fruits": [{"color": "green"}, {"length": 5}]}, None)
-
-        def using_dicts():
-            root = {
-                "fruits": [
-                    {"color": "green", "__typename": "Apple"},
-                    {"length": 5, "__typename": "Banana"},
-                ]
-            }
-
-            assert graphql_sync(schema, query, root) == expected
-
-        def using_objects():
-            class Apple:
-                __typename = "Apple"
-                color = "green"
-
-            class Banana:
-                __typename = "Banana"
-                length = 5
-
-            class Root:
-                fruits = [Apple(), Banana()]
-
-            assert graphql_sync(schema, query, Root()) == expected
-
-    def describe_specifying_interface_type_using_typename():
-        schema = build_schema(
-            """
-            type Query {
-              characters: [Character]
-            }
-
-            interface Character {
-              name: String!
-            }
-
-            type Human implements Character {
-              name: String!
-              totalCredits: Int
-            }
-
-            type Droid implements Character {
-              name: String!
-              primaryFunction: String
-            }
-            """
-        )
-
-        query = """
-            {
-              characters {
-                name
-                ... on Human {
-                  totalCredits
-                }
-                ... on Droid {
-                  primaryFunction
-                }
-              }
-            }
-            """
-
-        expected = (
-            {
-                "characters": [
-                    {"name": "Han Solo", "totalCredits": 10},
-                    {"name": "R2-D2", "primaryFunction": "Astromech"},
-                ]
-            },
-            None,
-        )
-
-        def using_dicts():
-            root = {
-                "characters": [
-                    {"name": "Han Solo", "totalCredits": 10, "__typename": "Human"},
-                    {
-                        "name": "R2-D2",
-                        "primaryFunction": "Astromech",
-                        "__typename": "Droid",
-                    },
-                ]
-            }
-
-            assert graphql_sync(schema, query, root) == expected
-
-        def using_objects():
-            class Human:
-                __typename = "Human"
-                name = "Han Solo"
-                totalCredits = 10
-
-            class Droid:
-                __typename = "Droid"
-                name = "R2-D2"
-                primaryFunction = "Astromech"
-
-            class Root:
-                characters = [Human(), Droid()]
-
-            assert graphql_sync(schema, query, Root()) == expected
-
     def custom_scalar():
         sdl = dedent(
             """
@@ -643,16 +599,34 @@ def describe_schema_builder():
     def unreferenced_type_implementing_referenced_interface():
         sdl = dedent(
             """
-            type Concrete implements Iface {
+            type Concrete implements Interface {
               key: String
             }
 
-            interface Iface {
+            interface Interface {
               key: String
             }
 
             type Query {
-              iface: Iface
+              interface: Interface
+            }
+            """
+        )
+        assert cycle_sdl(sdl) == sdl
+
+    def unreferenced_interface_implementing_referenced_interface():
+        sdl = dedent(
+            """
+            interface Child implements Parent {
+              key: String
+            }
+
+            interface Parent {
+              key: String
+            }
+
+            type Query {
+              interfaceField: Parent
             }
             """
         )
@@ -683,12 +657,21 @@ def describe_schema_builder():
               OTHER_VALUE @deprecated(reason: "Terrible reasons")
             }
 
+            input MyInput {
+              oldInput: String @deprecated
+              otherInput: String @deprecated(reason: "Use newInput")
+              newInput: String
+            }
+
             type Query {
               field1: String @deprecated
               field2: Int @deprecated(reason: "Because I said so")
               enum: MyEnum
+              field3(oldArg: String @deprecated, arg: String): String
+              field4(oldArg: String @deprecated(reason: "Why not?"), arg: String): String
+              field5(arg: MyInput): String
             }
-            """
+            """  # noqa: E501
         )
         assert cycle_sdl(sdl) == sdl
 
@@ -715,6 +698,232 @@ def describe_schema_builder():
         assert field2.is_deprecated is True
         assert field2.deprecation_reason == "Because I said so"
 
+        input_fields = assert_input_object_type(schema.get_type("MyInput")).fields
+
+        new_input = input_fields["newInput"]
+        assert new_input.deprecation_reason is None
+
+        old_input = input_fields["oldInput"]
+        assert old_input.deprecation_reason == "No longer supported"
+
+        other_input = input_fields["otherInput"]
+        assert other_input.deprecation_reason == "Use newInput"
+
+        field3_old_arg = root_fields["field3"].args["oldArg"]
+        assert field3_old_arg.deprecation_reason == "No longer supported"
+
+        field4_old_arg = root_fields["field4"].args["oldArg"]
+        assert field4_old_arg.deprecation_reason == "Why not?"
+
+    def supports_specified_by_directives():
+        sdl = dedent(
+            """
+            scalar Foo @specifiedBy(url: "https://example.com/foo_spec")
+
+            type Query {
+              foo: Foo @deprecated
+            }
+            """
+        )
+        assert cycle_sdl(sdl) == sdl
+
+        schema = build_schema(sdl)
+
+        foo_scalar = assert_scalar_type(schema.get_type("Foo"))
+        assert foo_scalar.specified_by_url == "https://example.com/foo_spec"
+
+    def correctly_extend_scalar_type():
+        scalar_sdl = dedent(
+            """
+            scalar SomeScalar
+
+            extend scalar SomeScalar @foo
+
+            extend scalar SomeScalar @bar
+            """
+        )
+        schema = build_schema(
+            scalar_sdl
+            + dedent(
+                """
+                directive @foo on SCALAR
+                directive @bar on SCALAR
+                """
+            )
+        )
+
+        some_scalar = assert_scalar_type(schema.get_type("SomeScalar"))
+        assert print_type(some_scalar) + "\n" == dedent(
+            """
+            scalar SomeScalar
+            """
+        )
+
+        assert print_all_ast_nodes(some_scalar) == scalar_sdl
+
+    def correctly_extend_object_type():
+        object_sdl = dedent(
+            """
+            type SomeObject implements Foo {
+              first: String
+            }
+
+            extend type SomeObject implements Bar {
+              second: Int
+            }
+
+            extend type SomeObject implements Baz {
+              third: Float
+            }
+            """
+        )
+        schema = build_schema(
+            object_sdl
+            + dedent(
+                """
+                interface Foo
+                interface Bar
+                interface Baz
+                """
+            )
+        )
+
+        some_object = assert_object_type(schema.get_type("SomeObject"))
+        assert print_type(some_object) + "\n" == dedent(
+            """
+            type SomeObject implements Foo & Bar & Baz {
+              first: String
+              second: Int
+              third: Float
+            }
+            """
+        )
+
+        assert print_all_ast_nodes(some_object) == object_sdl
+
+    def correctly_extend_interface_type():
+        interface_sdl = dedent(
+            """
+            interface SomeInterface {
+              first: String
+            }
+
+            extend interface SomeInterface {
+              second: Int
+            }
+
+            extend interface SomeInterface {
+              third: Float
+            }
+            """
+        )
+        schema = build_schema(interface_sdl)
+
+        some_interface = assert_interface_type(schema.get_type("SomeInterface"))
+        assert print_type(some_interface) + "\n" == dedent(
+            """
+            interface SomeInterface {
+              first: String
+              second: Int
+              third: Float
+            }
+            """
+        )
+
+        assert print_all_ast_nodes(some_interface) == interface_sdl
+
+    def correctly_extend_union_type():
+        union_sdl = dedent(
+            """
+            union SomeUnion = FirstType
+
+            extend union SomeUnion = SecondType
+
+            extend union SomeUnion = ThirdType
+            """
+        )
+        schema = build_schema(
+            union_sdl
+            + dedent(
+                """
+                type FirstType
+                type SecondType
+                type ThirdType
+                """
+            )
+        )
+
+        some_union = assert_union_type(schema.get_type("SomeUnion"))
+        assert print_type(some_union) + "\n" == dedent(
+            """
+            union SomeUnion = FirstType | SecondType | ThirdType
+            """
+        )
+
+        assert print_all_ast_nodes(some_union) == union_sdl
+
+    def correctly_extend_enum_type():
+        enum_sdl = dedent(
+            """
+            enum SomeEnum {
+              FIRST
+            }
+
+            extend enum SomeEnum {
+              SECOND
+            }
+
+            extend enum SomeEnum {
+              THIRD
+            }
+            """
+        )
+        schema = build_schema(enum_sdl)
+
+        some_enum = assert_enum_type(schema.get_type("SomeEnum"))
+        assert print_type(some_enum) + "\n" == dedent(
+            """
+            enum SomeEnum {
+              FIRST
+              SECOND
+              THIRD
+            }
+            """
+        )
+
+        assert print_all_ast_nodes(some_enum) == enum_sdl
+
+    def correctly_extend_input_object_type():
+        input_sdl = dedent(
+            """
+            input SomeInput {
+              first: String
+            }
+
+            extend input SomeInput {
+              second: Int
+            }
+
+            extend input SomeInput {
+              third: Float
+            }
+            """
+        )
+        schema = build_schema(input_sdl)
+
+        some_input = assert_input_object_type(schema.get_type("SomeInput"))
+        assert print_type(some_input) + "\n" == dedent(
+            """
+            input SomeInput {
+              first: String
+              second: Int
+              third: Float
+            }
+            """
+        )
+
+        assert print_all_ast_nodes(some_input) == input_sdl
+
     def correctly_assign_ast_nodes():
         sdl = dedent(
             """
@@ -761,21 +970,17 @@ def describe_schema_builder():
         test_scalar = assert_scalar_type(schema.get_type("TestScalar"))
         test_directive = assert_directive(schema.get_directive("test"))
 
-        restored_schema_ast = DocumentNode(
-            definitions=[
-                schema.ast_node,
-                query.ast_node,
-                test_input.ast_node,
-                test_enum.ast_node,
-                test_union.ast_node,
-                test_interface.ast_node,
-                test_type.ast_node,
-                test_scalar.ast_node,
-                test_directive.ast_node,
-            ],
-            loc=None,
-        )
-        assert restored_schema_ast == ast
+        assert [
+            schema.ast_node,
+            query.ast_node,
+            test_input.ast_node,
+            test_enum.ast_node,
+            test_union.ast_node,
+            test_interface.ast_node,
+            test_type.ast_node,
+            test_scalar.ast_node,
+            test_directive.ast_node,
+        ] == ast.definitions
 
         test_field = query.fields["testField"]
         assert print_ast_node(test_field) == (
@@ -836,6 +1041,35 @@ def describe_schema_builder():
         errors = validate_schema(schema)
         assert errors
 
+    def do_not_override_standard_types():
+        # Note: not sure it's desired behaviour to just silently ignore override
+        # attempts so just documenting it here.
+
+        schema = build_schema(
+            """
+            scalar ID
+
+            scalar __Schema
+            """
+        )
+
+        assert schema.get_type("ID") is GraphQLID
+        assert schema.get_type("__Schema") is introspection_types["__Schema"]
+
+    def allows_to_reference_introspection_types():
+        schema = build_schema(
+            """
+            type Query {
+              introspectionField: __EnumValue
+            }
+            """
+        )
+
+        query_type = assert_object_type(schema.get_type("Query"))
+        __EnumValue = introspection_types["__EnumValue"]
+        assert query_type.fields["introspectionField"].type is __EnumValue
+        assert schema.get_type("__EnumValue") is introspection_types["__EnumValue"]
+
     def rejects_invalid_sdl():
         sdl = """
             type Query {
@@ -844,8 +1078,7 @@ def describe_schema_builder():
             """
         with raises(TypeError) as exc_info:
             build_schema(sdl)
-        msg = str(exc_info.value)
-        assert msg == "Unknown directive 'unknown'."
+        assert str(exc_info.value) == "Unknown directive '@unknown'."
 
     def allows_to_disable_sdl_validation():
         sdl = """
@@ -855,3 +1088,21 @@ def describe_schema_builder():
             """
         build_schema(sdl, assume_valid=True)
         build_schema(sdl, assume_valid_sdl=True)
+
+    def throws_on_unknown_types():
+        sdl = """
+            type Query {
+              unknown: UnknownType
+            }
+            """
+        with raises(TypeError) as exc_info:
+            build_schema(sdl, assume_valid_sdl=True)
+        assert str(exc_info.value).endswith("Unknown type: 'UnknownType'.")
+
+    def rejects_invalid_ast():
+        with raises(TypeError) as exc_info:
+            build_ast_schema(None)  # type: ignore
+        assert str(exc_info.value) == "Must provide valid Document AST."
+        with raises(TypeError) as exc_info:
+            build_ast_schema({})  # type: ignore
+        assert str(exc_info.value) == "Must provide valid Document AST."
