@@ -1,7 +1,6 @@
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql import graphql_sync
-from graphql.pyutils import dedent
 from graphql.type import (
     GraphQLArgument,
     GraphQLBoolean,
@@ -23,21 +22,24 @@ from graphql.utilities import (
     print_schema,
 )
 
+from ..utils import dedent
+
 
 def cycle_introspection(sdl_string):
     """Test that the client side introspection gives the same result.
 
     This function does a full cycle of going from a string with the contents of the SDL,
     build in-memory GraphQLSchema from it, produce a client-side representation of the
-    schema by using "build_client_schema" and then finally printing that that schema
-    into the SDL.
+    schema by using "build_client_schema" and then return that schema printed as SDL.
     """
+    options = dict(specified_by_url=True, directive_is_repeatable=True)
+
     server_schema = build_schema(sdl_string)
-    initial_introspection = introspection_from_schema(server_schema)
+    initial_introspection = introspection_from_schema(server_schema, **options)
     client_schema = build_client_schema(initial_introspection)
     # If the client then runs the introspection query against the client-side schema,
     # it should get a result identical to what was returned by the server
-    second_introspection = introspection_from_schema(client_schema)
+    second_introspection = introspection_from_schema(client_schema, **options)
 
     # If the client then runs the introspection query against the client-side
     # schema, it should get a result identical to what was returned by the server.
@@ -49,11 +51,12 @@ def describe_type_system_build_schema_from_introspection():
     def builds_a_simple_schema():
         sdl = dedent(
             '''
+            """Simple schema"""
             schema {
               query: Simple
             }
 
-            """This is simple type"""
+            """This is a simple type"""
             type Simple {
               """This is a string field"""
               string: String
@@ -144,7 +147,7 @@ def describe_type_system_build_schema_from_introspection():
         custom_scalar = schema.get_type("CustomScalar")
         assert client_schema.get_type("CustomScalar") is not custom_scalar
 
-    def include_standard_type_only_if_it_is_used():
+    def includes_standard_types_only_if_they_are_used():
         schema = build_schema(
             """
             type Query {
@@ -218,6 +221,37 @@ def describe_type_system_build_schema_from_introspection():
 
         assert cycle_introspection(sdl) == sdl
 
+    def builds_a_schema_with_an_interface_hierarchy():
+        sdl = dedent(
+            '''
+            type Dog implements Friendly & Named {
+              bestFriend: Friendly
+              name: String
+            }
+
+            interface Friendly implements Named {
+              """The best friend of this friendly thing"""
+              bestFriend: Friendly
+              name: String
+            }
+
+            type Human implements Friendly & Named {
+              bestFriend: Friendly
+              name: String
+            }
+
+            interface Named {
+              name: String
+            }
+
+            type Query {
+              friendly: Friendly
+            }
+            '''
+        )
+
+        assert cycle_introspection(sdl) == sdl
+
     def builds_a_schema_with_an_implicit_interface():
         sdl = dedent(
             '''
@@ -318,10 +352,8 @@ def describe_type_system_build_schema_from_introspection():
                 "VEGETABLES": GraphQLEnumValue(
                     1, description="Foods that are vegetables."
                 ),
-                "FRUITS": GraphQLEnumValue(2, description="Foods that are fruits."),
-                "OILS": GraphQLEnumValue(3, description="Foods that are oils."),
-                "DAIRY": GraphQLEnumValue(4, description="Foods that are dairy."),
-                "MEAT": GraphQLEnumValue(5, description="Foods that are meat."),
+                "FRUITS": GraphQLEnumValue(2),
+                "OILS": GraphQLEnumValue(3, deprecation_reason="Too fatty."),
             },
             description="Varieties of food stuffs",
         )
@@ -352,21 +384,34 @@ def describe_type_system_build_schema_from_introspection():
         # It's also an Enum type on the client.
         client_food_enum = assert_enum_type(client_schema.get_type("Food"))
 
-        values_dict = client_food_enum.values
-        descriptions = {name: value.description for name, value in values_dict.items()}
-        assert descriptions == {
-            "VEGETABLES": "Foods that are vegetables.",
-            "FRUITS": "Foods that are fruits.",
-            "OILS": "Foods that are oils.",
-            "DAIRY": "Foods that are dairy.",
-            "MEAT": "Foods that are meat.",
+        # Client types do not get server-only values, so they are set to None
+        # rather than using the integers defined in the "server" schema.
+        values = {
+            name: value.to_kwargs() for name, value in client_food_enum.values.items()
+        }
+        assert values == {
+            "VEGETABLES": {
+                "value": None,
+                "description": "Foods that are vegetables.",
+                "deprecation_reason": None,
+                "extensions": None,
+                "ast_node": None,
+            },
+            "FRUITS": {
+                "value": None,
+                "description": None,
+                "deprecation_reason": None,
+                "extensions": None,
+                "ast_node": None,
+            },
+            "OILS": {
+                "value": None,
+                "description": None,
+                "deprecation_reason": "Too fatty.",
+                "extensions": None,
+                "ast_node": None,
+            },
         }
-        values = values_dict.values()
-        assert all(value.value is None for value in values)
-        assert all(value.is_deprecated is False for value in values)
-        assert all(value.deprecation_reason is None for value in values)
-        assert all(value.extensions is None for value in values)
-        assert all(value.ast_node is None for value in values)
 
     def builds_a_schema_with_an_input_object():
         sdl = dedent(
@@ -419,7 +464,7 @@ def describe_type_system_build_schema_from_introspection():
         sdl = dedent(
             '''
             """This is a custom directive"""
-            directive @customDirective on FIELD
+            directive @customDirective repeatable on FIELD
 
             type Query {
               string: String
@@ -478,6 +523,34 @@ def describe_type_system_build_schema_from_introspection():
 
         assert cycle_introspection(sdl) == sdl
 
+    def builds_a_schema_with_empty_deprecation_reasons():
+        sdl = dedent(
+            """
+            type Query {
+              someField: String @deprecated(reason: "")
+            }
+
+            enum SomeEnum {
+              SOME_VALUE @deprecated(reason: "")
+            }
+            """
+        )
+
+        assert cycle_introspection(sdl) == sdl
+
+    def builds_a_schema_with_specified_by_url():
+        sdl = dedent(
+            """
+            scalar Foo @specifiedBy(url: "https://example.com/foo_spec")
+
+            type Query {
+              foo: Foo
+            }
+            """
+        )
+
+        assert cycle_introspection(sdl) == sdl
+
     def can_use_client_schema_for_limited_execution():
         schema = build_schema(
             """
@@ -505,6 +578,14 @@ def describe_type_system_build_schema_from_introspection():
 
         assert result.data == {"foo": "bar"}
 
+    def can_build_invalid_schema():
+        schema = build_schema("type Query", assume_valid=True)
+
+        introspection = introspection_from_schema(schema)
+        client_schema = build_client_schema(introspection, assume_valid=True)
+
+        assert client_schema.to_kwargs()["assume_valid"] is True
+
     def describe_throws_when_given_invalid_introspection():
         dummy_schema = build_schema(
             """
@@ -512,6 +593,10 @@ def describe_type_system_build_schema_from_introspection():
               foo(bar: String): String
             }
 
+            interface SomeInterface {
+              foo: String
+            }
+
             union SomeUnion = Query
 
             enum SomeEnum { FOO }
@@ -532,7 +617,7 @@ def describe_type_system_build_schema_from_introspection():
             assert str(exc_info.value) == (
                 "Invalid or incomplete introspection result. Ensure that you"
                 " are passing the 'data' attribute of an introspection response"
-                " and no 'errors' were returned alongside: None"
+                " and no 'errors' were returned alongside: None."
             )
 
             with raises(TypeError) as exc_info:
@@ -542,7 +627,7 @@ def describe_type_system_build_schema_from_introspection():
             assert str(exc_info.value) == (
                 "Invalid or incomplete introspection result. Ensure that you"
                 " are passing the 'data' attribute of an introspection response"
-                " and no 'errors' were returned alongside: {}"
+                " and no 'errors' were returned alongside: {}."
             )
 
         def throws_when_referenced_unknown_type():
@@ -596,94 +681,105 @@ def describe_type_system_build_schema_from_introspection():
             with raises(TypeError) as exc_info:
                 build_client_schema(introspection)
 
-            assert str(exc_info.value) == "Unknown type reference: {}"
+            assert str(exc_info.value) == "Unknown type reference: {}."
 
         def throws_when_missing_kind():
             introspection = introspection_from_schema(dummy_schema)
 
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
             assert query_type_introspection["kind"] == "OBJECT"
             del query_type_introspection["kind"]
 
-            with raises(TypeError) as exc_info:
-                build_client_schema(introspection)
-
-            assert str(exc_info.value).startswith(
-                "Invalid or incomplete introspection result."
+            with raises(
+                TypeError,
+                match=r"^Invalid or incomplete introspection result\."
                 " Ensure that a full introspection query is used"
-                " in order to build a client schema: {'name': 'Query',"
-            )
+                r" in order to build a client schema: {'name': 'Query', .*}\.$",
+            ):
+                build_client_schema(introspection)
 
         def throws_when_missing_interfaces():
             introspection = introspection_from_schema(dummy_schema)
 
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
             assert query_type_introspection["interfaces"] == []
             del query_type_introspection["interfaces"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Query interfaces cannot be resolved."
+                " Introspection result missing interfaces:"
+                r" {'kind': 'OBJECT', 'name': 'Query', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing interfaces:"
-                " {'kind': 'OBJECT', 'name': 'Query',"
+        def legacy_support_for_interfaces_with_null_as_interfaces_field():
+            introspection = introspection_from_schema(dummy_schema)
+            some_interface_introspection = next(
+                type_
+                for type_ in introspection["__schema"]["types"]
+                if type_["name"] == "SomeInterface"
             )
 
+            assert some_interface_introspection["interfaces"] == []
+            some_interface_introspection["interfaces"] = None
+
+            client_schema = build_client_schema(introspection)
+            assert print_schema(client_schema) == print_schema(dummy_schema)
+
         def throws_when_missing_fields():
             introspection = introspection_from_schema(dummy_schema)
-
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
+
             assert query_type_introspection["fields"]
             del query_type_introspection["fields"]
 
-            with raises(TypeError) as exc_info:
-                build_client_schema(introspection)
-
-            assert str(exc_info.value).startswith(
-                "Query fields cannot be resolved:"
+            with raises(
+                TypeError,
+                match="^Query fields cannot be resolved."
                 " Introspection result missing fields:"
-                " {'kind': 'OBJECT', 'name': 'Query',"
-            )
+                r" {'kind': 'OBJECT', 'name': 'Query', .*}\.$",
+            ):
+                build_client_schema(introspection)
 
         def throws_when_missing_field_args():
             introspection = introspection_from_schema(dummy_schema)
 
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
             assert query_type_introspection["fields"][0]["args"]
             del query_type_introspection["fields"][0]["args"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Query fields cannot be resolved."
+                r" Introspection result missing field args: {'name': 'foo', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Query fields cannot be resolved:"
-                " Introspection result missing field args: {'name': 'foo',"
-            )
-
         def throws_when_output_type_is_used_as_an_arg_type():
             introspection = introspection_from_schema(dummy_schema)
 
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
             assert (
                 query_type_introspection["fields"][0]["args"][0]["type"]["name"]
                 == "String"
@@ -696,19 +792,44 @@ def describe_type_system_build_schema_from_introspection():
                 build_client_schema(introspection)
 
             assert str(exc_info.value).startswith(
-                "Query fields cannot be resolved:"
+                "Query fields cannot be resolved."
                 " Introspection must provide input type for arguments,"
                 " but received: SomeUnion."
             )
 
+        def throws_when_output_type_is_used_as_an_input_value_type():
+            introspection = introspection_from_schema(dummy_schema)
+
+            input_object_type_introspection = next(
+                type_
+                for type_ in introspection["__schema"]["types"]
+                if type_["name"] == "SomeInputObject"
+            )
+            assert (
+                input_object_type_introspection["inputFields"][0]["type"]["name"]
+                == "String"
+            )
+            input_object_type_introspection["inputFields"][0]["type"][
+                "name"
+            ] = "SomeUnion"
+
+            with raises(TypeError) as exc_info:
+                build_client_schema(introspection)
+
+            assert str(exc_info.value).startswith(
+                "SomeInputObject fields cannot be resolved."
+                " Introspection must provide input type for input fields,"
+                " but received: SomeUnion."
+            )
+
         def throws_when_input_type_is_used_as_a_field_type():
             introspection = introspection_from_schema(dummy_schema)
 
-            query_type_introspection = [
+            query_type_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "Query"
-            ][0]
+            )
             assert query_type_introspection["fields"][0]["type"]["name"] == "String"
             query_type_introspection["fields"][0]["type"]["name"] = "SomeInputObject"
 
@@ -716,7 +837,7 @@ def describe_type_system_build_schema_from_introspection():
                 build_client_schema(introspection)
 
             assert str(exc_info.value).startswith(
-                "Query fields cannot be resolved:"
+                "Query fields cannot be resolved."
                 " Introspection must provide output type for fields,"
                 " but received: SomeInputObject."
             )
@@ -724,60 +845,57 @@ def describe_type_system_build_schema_from_introspection():
         def throws_when_missing_possible_types():
             introspection = introspection_from_schema(dummy_schema)
 
-            some_union_introspection = [
+            some_union_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "SomeUnion"
-            ][0]
+            )
             assert some_union_introspection["possibleTypes"]
             del some_union_introspection["possibleTypes"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Introspection result missing possibleTypes:"
+                r" {'kind': 'UNION', 'name': 'SomeUnion', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing possibleTypes:"
-                " {'kind': 'UNION', 'name': 'SomeUnion',"
-            )
-
         def throws_when_missing_enum_values():
             introspection = introspection_from_schema(dummy_schema)
 
-            some_enum_introspection = [
+            some_enum_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "SomeEnum"
-            ][0]
+            )
             assert some_enum_introspection["enumValues"]
             del some_enum_introspection["enumValues"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Introspection result missing enumValues:"
+                r" {'kind': 'ENUM', 'name': 'SomeEnum', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing enumValues:"
-                " {'kind': 'ENUM', 'name': 'SomeEnum',"
-            )
-
         def throws_when_missing_input_fields():
             introspection = introspection_from_schema(dummy_schema)
 
-            some_input_object_introspection = [
+            some_input_object_introspection = next(
                 type_
                 for type_ in introspection["__schema"]["types"]
                 if type_["name"] == "SomeInputObject"
-            ][0]
+            )
             assert some_input_object_introspection["inputFields"]
             del some_input_object_introspection["inputFields"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Introspection result missing inputFields:"
+                r" {'kind': 'INPUT_OBJECT', 'name': 'SomeInputObject', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing inputFields:"
-                " {'kind': 'INPUT_OBJECT', 'name': 'SomeInputObject',"
-            )
-
         def throws_when_missing_directive_locations():
             introspection = introspection_from_schema(dummy_schema)
 
@@ -786,14 +904,13 @@ def describe_type_system_build_schema_from_introspection():
             assert some_directive_introspection["locations"] == ["QUERY"]
             del some_directive_introspection["locations"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Introspection result missing directive locations:"
+                r" {'name': 'SomeDirective', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing directive locations:"
-                " {'name': 'SomeDirective',"
-            )
-
         def throws_when_missing_directive_args():
             introspection = introspection_from_schema(dummy_schema)
 
@@ -802,14 +919,13 @@ def describe_type_system_build_schema_from_introspection():
             assert some_directive_introspection["args"] == []
             del some_directive_introspection["args"]
 
-            with raises(TypeError) as exc_info:
+            with raises(
+                TypeError,
+                match="^Introspection result missing directive args:"
+                r" {'name': 'SomeDirective', .*}\.$",
+            ):
                 build_client_schema(introspection)
 
-            assert str(exc_info.value).startswith(
-                "Introspection result missing directive args:"
-                " {'name': 'SomeDirective',"
-            )
-
     def describe_very_deep_decorators_are_not_supported():
         def fails_on_very_deep_lists_more_than_7_levels():
             schema = build_schema(
@@ -826,7 +942,7 @@ def describe_type_system_build_schema_from_introspection():
                 build_client_schema(introspection)
 
             assert str(exc_info.value) == (
-                "Query fields cannot be resolved:"
+                "Query fields cannot be resolved."
                 " Decorated type deeper than introspection query."
             )
 
@@ -845,7 +961,7 @@ def describe_type_system_build_schema_from_introspection():
                 build_client_schema(introspection)
 
             assert str(exc_info.value) == (
-                "Query fields cannot be resolved:"
+                "Query fields cannot be resolved."
                 " Decorated type deeper than introspection query."
             )
 
@@ -875,19 +991,22 @@ def describe_type_system_build_schema_from_introspection():
             schema = build_schema(sdl, assume_valid=True)
             introspection = introspection_from_schema(schema)
 
-            foo_type_introspection = introspection["__schema"]["types"][1]
-            assert foo_type_introspection["name"] == "Foo"
-            assert foo_type_introspection["interfaces"] == []
+            foo_introspection = next(
+                type_
+                for type_ in introspection["__schema"]["types"]
+                if type_["name"] == "Foo"
+            )
+            assert foo_introspection["interfaces"] == []
             # we need to patch here since invalid interfaces cannot be built with Python
-            foo_type_introspection["interfaces"] = [
+            foo_introspection["interfaces"] = [
                 {"kind": "OBJECT", "name": "Foo", "ofType": None}
             ]
 
             with raises(TypeError) as exc_info:
                 build_client_schema(introspection)
             assert str(exc_info.value) == (
-                "Foo interfaces cannot be resolved: "
-                "Expected Foo to be a GraphQL Interface type."
+                "Foo interfaces cannot be resolved."
+                " Expected Foo to be a GraphQL Interface type."
             )
 
         def recursive_union():
@@ -901,18 +1020,21 @@ def describe_type_system_build_schema_from_introspection():
             schema = build_schema(sdl, assume_valid=True)
             introspection = introspection_from_schema(schema)
 
-            foo_type_introspection = introspection["__schema"]["types"][1]
-            assert foo_type_introspection["name"] == "Foo"
-            assert foo_type_introspection["kind"] == "UNION"
-            assert foo_type_introspection["possibleTypes"] == []
+            foo_introspection = next(
+                type_
+                for type_ in introspection["__schema"]["types"]
+                if type_["name"] == "Foo"
+            )
+            assert foo_introspection["kind"] == "UNION"
+            assert foo_introspection["possibleTypes"] == []
             # we need to patch here since invalid unions cannot be built with Python
-            foo_type_introspection["possibleTypes"] = [
+            foo_introspection["possibleTypes"] = [
                 {"kind": "UNION", "name": "Foo", "ofType": None}
             ]
 
             with raises(TypeError) as exc_info:
                 build_client_schema(introspection)
             assert str(exc_info.value) == (
-                "Foo types cannot be resolved:"
+                "Foo types cannot be resolved."
                 " Expected Foo to be a GraphQL Object type."
             )
