@@ -1,7 +1,7 @@
 from math import nan
 from typing import Any, List, NamedTuple, Union
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql.error import GraphQLError
 from graphql.pyutils import Undefined
@@ -10,6 +10,7 @@ from graphql.type import (
     GraphQLFloat,
     GraphQLInputField,
     GraphQLInputObjectType,
+    GraphQLInputType,
     GraphQLInt,
     GraphQLList,
     GraphQLNonNull,
@@ -39,7 +40,7 @@ def expect_errors(result: CoercedValue) -> List[CoercedValueError]:
 
 
 def describe_coerce_input_value():
-    def _coerce_value(input_value, type_):
+    def _coerce_value(input_value: Any, type_: GraphQLInputType):
         errors: List[CoercedValueError] = []
         append = errors.append
 
@@ -168,14 +169,26 @@ def describe_coerce_input_value():
         def returns_an_error_for_an_invalid_field():
             result = _coerce_value({"foo": nan}, TestInputObject)
             assert expect_errors(result) == [
-                ("Int cannot represent non-integer value: nan", ["foo"], nan,)
+                (
+                    "Int cannot represent non-integer value: nan",
+                    ["foo"],
+                    nan,
+                )
             ]
 
         def returns_multiple_errors_for_multiple_invalid_fields():
             result = _coerce_value({"foo": "abc", "bar": "def"}, TestInputObject)
             assert expect_errors(result) == [
-                ("Int cannot represent non-integer value: 'abc'", ["foo"], "abc",),
-                ("Int cannot represent non-integer value: 'def'", ["bar"], "def",),
+                (
+                    "Int cannot represent non-integer value: 'abc'",
+                    ["foo"],
+                    "abc",
+                ),
+                (
+                    "Int cannot represent non-integer value: 'def'",
+                    ["bar"],
+                    "def",
+                ),
             ]
 
         def returns_error_for_a_missing_required_field():
@@ -275,8 +288,16 @@ def describe_coerce_input_value():
         def returns_an_error_for_an_invalid_input():
             result = _coerce_value([1, "b", True, 4], TestList)
             assert expect_errors(result) == [
-                ("Int cannot represent non-integer value: 'b'", [1], "b",),
-                ("Int cannot represent non-integer value: True", [2], True,),
+                (
+                    "Int cannot represent non-integer value: 'b'",
+                    [1],
+                    "b",
+                ),
+                (
+                    "Int cannot represent non-integer value: True",
+                    [2],
+                    True,
+                ),
             ]
 
         def returns_a_list_for_a_non_list_value():
