@@ -1,16 +1,14 @@
 from json import dumps
 from typing import Optional
 
-from pytest import raises  # type: ignore
+from pytest import raises
 
 from graphql.error import GraphQLSyntaxError
 from graphql.language import Lexer, Source, TokenKind, parse
-from graphql.pyutils import dedent
 from graphql.utilities import strip_ignored_characters
 
-# noinspection PyUnresolvedReferences
 from ..fixtures import kitchen_sink_query, kitchen_sink_sdl  # noqa: F401
-
+from ..utils import dedent
 
 ignored_tokens = [
     # UnicodeBOM
@@ -77,21 +75,6 @@ class ExpectStripped:
 
 
 def describe_strip_ignored_characters():
-    def asserts_that_a_source_was_provided():
-        with raises(
-            TypeError, match="missing 1 required positional argument: 'source'"
-        ):
-            # noinspection PyArgumentList
-            strip_ignored_characters()  # type: ignore
-        with raises(TypeError, match="Must provide string or Source. Received: None"):
-            # noinspection PyTypeChecker
-            strip_ignored_characters(None)  # type: ignore
-
-    def asserts_that_a_valid_source_was_provided():
-        with raises(TypeError, match="Must provide string or Source. Received: {}"):
-            # noinspection PyTypeChecker
-            strip_ignored_characters({})  # type: ignore
-
     def strips_ignored_characters_from_graphql_query_document():
         query = dedent(
             """
@@ -186,7 +169,7 @@ def describe_strip_ignored_characters():
             ExpectStripped("".join(ignored_tokens) + token).to_equal(token)
             ExpectStripped(token + "".join(ignored_tokens)).to_equal(token)
 
-    def strips_ignored_tokens_beetween_punctuator_tokens():
+    def strips_ignored_tokens_between_punctuator_tokens():
         ExpectStripped("[,)").to_equal("[)")
         ExpectStripped("[\r)").to_equal("[)")
         ExpectStripped("[\r\r)").to_equal("[)")
@@ -207,7 +190,7 @@ def describe_strip_ignored_characters():
                     left + right
                 )
 
-    def strips_ignored_tokens_beetween_punctuator_and_non_punctuator_tokens():
+    def strips_ignored_tokens_between_punctuator_and_non_punctuator_tokens():
         ExpectStripped("[,1").to_equal("[1")
         ExpectStripped("[\r1").to_equal("[1")
         ExpectStripped("[\r\r1").to_equal("[1")
@@ -342,13 +325,11 @@ def describe_strip_ignored_characters():
         # noinspection PyShadowingNames
         def expect_stripped_string(block_str: str):
             original_value = lex_value(block_str)
-
-            stripped_str = strip_ignored_characters(block_str)
-            stripped_value = lex_value(stripped_str)
+            stripped_value = lex_value(strip_ignored_characters(block_str))
 
             assert original_value == stripped_value, dedent(
                 f"""
-                Expected lexOne(stripIgnoredCharacters({block_str!r})
+                Expected lexValue(stripIgnoredCharacters({block_str!r})
                   to equal {original_value!r}
                   but got {stripped_value!r}
                 """
@@ -376,41 +357,6 @@ def describe_strip_ignored_characters():
         expect_stripped_string('"""\n a\n b"""').to_equal('"""a\nb"""')
         expect_stripped_string('"""\na\n b\nc"""').to_equal('"""a\n b\nc"""')
 
-        # Testing with length >5 is taking exponentially more time. However it is
-        # highly recommended to test with increased limit if you make any change.
-        max_combination_length = 5
-        possible_chars = ["\n", " ", '"', "a", "\\"]
-        num_possible_chars = len(possible_chars)
-        num_combinations = 1
-        for length in range(1, max_combination_length):
-            num_combinations *= num_possible_chars
-            for combination in range(num_combinations):
-                test_str = '"""'
-
-                left_over = combination
-                for i in range(length):
-                    reminder = left_over % num_possible_chars
-                    test_str += possible_chars[reminder]
-                    left_over = (left_over - reminder) // num_possible_chars
-
-                test_str += '"""'
-
-                try:
-                    test_value = lex_value(test_str)
-                except (AssertionError, GraphQLSyntaxError):
-                    continue  # skip invalid values
-
-                stripped_str = strip_ignored_characters(test_str)
-                stripped_value = lex_value(stripped_str)
-
-                assert test_value == stripped_value, dedent(
-                    f"""
-                    Expected lexOne(stripIgnoredCharacters({test_str!r})
-                      to equal {test_value!r}
-                      but got {stripped_value!r}
-                    """
-                )
-
     # noinspection PyShadowingNames
     def strips_kitchen_sink_query_but_maintains_the_exact_same_ast(
         kitchen_sink_query,  # noqa: F811
