@@ -2,9 +2,6 @@ from functools import partial
 
 from graphql.utilities import build_schema
 from graphql.validation import OverlappingFieldsCanBeMergedRule
-from graphql.validation.rules.overlapping_fields_can_be_merged import (
-    fields_conflict_message,
-)
 
 from .harness import assert_validation_errors
 
@@ -13,10 +10,6 @@ assert_errors = partial(assert_validation_errors, OverlappingFieldsCanBeMergedRu
 assert_valid = partial(assert_errors, errors=[])
 
 
-def assert_valid_with_schema(schema, query_str):
-    assert_errors(query_str, [], schema)
-
-
 def describe_validate_overlapping_fields_can_be_merged():
     def unique_fields():
         assert_valid(
@@ -101,9 +94,10 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "fido", "name and nickname are different fields"
-                    ),
+                    "message": "Fields 'fido' conflict"
+                    " because 'name' and 'nickname' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 15)],
                     "path": None,
                 }
@@ -134,9 +128,10 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "name", "nickname and name are different fields"
-                    ),
+                    "message": "Fields 'name' conflict"
+                    " because 'nickname' and 'name' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 15)],
                 }
             ],
@@ -152,9 +147,10 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "doesKnowCommand", "they have differing arguments"
-                    ),
+                    "message": "Fields 'doesKnowCommand' conflict"
+                    " because they have differing arguments."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 15)],
                 }
             ],
@@ -170,15 +166,16 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "doesKnowCommand", "they have differing arguments"
-                    ),
+                    "message": "Fields 'doesKnowCommand' conflict"
+                    " because they have differing arguments."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 15)],
                 }
             ],
         )
 
-    def conflicting_args():
+    def conflicting_arg_values():
         assert_errors(
             """
             fragment conflictingArgs on Dog {
@@ -188,9 +185,29 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "doesKnowCommand", "they have differing arguments"
-                    ),
+                    "message": "Fields 'doesKnowCommand' conflict"
+                    " because they have differing arguments."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
+                    "locations": [(3, 15), (4, 15)],
+                }
+            ],
+        )
+
+    def conflicting_arg_names():
+        assert_errors(
+            """
+            fragment conflictingArgs on Dog {
+              isAtLocation(x: 0)
+              isAtLocation(y: 0)
+            }
+            """,
+            [
+                {
+                    "message": "Fields 'isAtLocation' conflict"
+                    " because they have differing arguments."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 15)],
                 }
             ],
@@ -228,9 +245,10 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "x", "a and b are different fields"
-                    ),
+                    "message": "Fields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(7, 15), (10, 15)],
                 }
             ],
@@ -263,21 +281,24 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "x", "a and b are different fields"
-                    ),
+                    "message": "Fields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(18, 15), (21, 15)],
                 },
                 {
-                    "message": fields_conflict_message(
-                        "x", "c and a are different fields"
-                    ),
+                    "message": "Fields 'x' conflict"
+                    " because 'c' and 'a' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(14, 17), (18, 15)],
                 },
                 {
-                    "message": fields_conflict_message(
-                        "x", "c and b are different fields"
-                    ),
+                    "message": "Fields 'x' conflict"
+                    " because 'c' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(14, 17), (21, 15)],
                 },
             ],
@@ -297,9 +318,11 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "field", [("x", "a and b are different fields")]
-                    ),
+                    "message": "Fields 'field' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 17), (6, 15), (7, 17)],
                 }
             ],
@@ -321,13 +344,13 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "field",
-                        [
-                            ("x", "a and b are different fields"),
-                            ("y", "c and d are different fields"),
-                        ],
-                    ),
+                    "message": "Fields 'field' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields"
+                    " and subfields 'y' conflict"
+                    " because 'c' and 'd' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(3, 15), (4, 17), (5, 17), (7, 15), (8, 17), (9, 17)],
                     "path": None,
                 }
@@ -352,10 +375,12 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "field",
-                        [("deepField", [("x", "a and b are different fields")])],
-                    ),
+                    "message": "Fields 'field' conflict"
+                    " because subfields 'deepField' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [
                         (3, 15),
                         (4, 17),
@@ -390,9 +415,11 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "deepField", [("x", "a and b are different fields")]
-                    ),
+                    "message": "Fields 'deepField' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(4, 17), (5, 19), (7, 17), (8, 19)],
                 }
             ],
@@ -427,9 +454,11 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "deeperField", [("x", "a and b are different fields")]
-                    ),
+                    "message": "Fields 'deeperField' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(12, 17), (13, 19), (15, 17), (16, 19)],
                 }
             ],
@@ -463,13 +492,13 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "field",
-                        [
-                            ("x", "a and b are different fields"),
-                            ("y", "c and d are different fields"),
-                        ],
-                    ),
+                    "message": "Fields 'field' conflict"
+                    " because subfields 'x' conflict"
+                    " because 'a' and 'b' are different fields"
+                    " and subfields 'y' conflict"
+                    " because 'c' and 'd' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [
                         (3, 15),
                         (11, 15),
@@ -586,9 +615,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "scalar", "they return conflicting types Int and String!"
-                        ),
+                        "message": "Fields 'scalar' conflict because"
+                        " they return conflicting types 'Int' and 'String!'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (8, 23)],
                     }
                 ],
@@ -635,9 +665,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "scalar", "they return conflicting types Int and String"
-                        ),
+                        "message": "Fields 'scalar' conflict because"
+                        " they return conflicting types 'Int' and 'String'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (8, 23)],
                     }
                 ],
@@ -692,15 +723,11 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "other",
-                            [
-                                (
-                                    "scalar",
-                                    "scalar and unrelatedField are different fields",
-                                )
-                            ],
-                        ),
+                        "message": "Fields 'other' conflict because"
+                        " subfields 'scalar' conflict because"
+                        " 'scalar' and 'unrelatedField' are different fields."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(31, 19), (39, 19), (34, 19), (42, 19)],
                         "path": None,
                     }
@@ -724,9 +751,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "scalar", "they return conflicting types String! and String"
-                        ),
+                        "message": "Fields 'scalar' conflict because"
+                        " they return conflicting types 'String!' and 'String'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (8, 23)],
                     }
                 ],
@@ -753,11 +781,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "box",
-                            "they return conflicting types"
-                            " [StringBox] and StringBox",
-                        ),
+                        "message": "Fields 'box' conflict because they return"
+                        " conflicting types '[StringBox]' and 'StringBox'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (10, 23)],
                     }
                 ],
@@ -783,11 +810,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "box",
-                            "they return conflicting types"
-                            " StringBox and [StringBox]",
-                        ),
+                        "message": "Fields 'box' conflict because they return"
+                        " conflicting types 'StringBox' and '[StringBox]'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (10, 23)],
                     }
                 ],
@@ -815,9 +841,10 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "val", "scalar and unrelatedField are different fields"
-                        ),
+                        "message": "Fields 'val' conflict because"
+                        " 'scalar' and 'unrelatedField' are different fields."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(6, 25), (7, 25)],
                     }
                 ],
@@ -844,15 +871,11 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "box",
-                            [
-                                (
-                                    "scalar",
-                                    "they return conflicting types String and Int",
-                                )
-                            ],
-                        ),
+                        "message": "Fields 'box' conflict"
+                        " because subfields 'scalar' conflict"
+                        " because they return conflicting types 'String' and 'Int'."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [(5, 23), (6, 25), (10, 23), (11, 25)],
                         "path": None,
                     }
@@ -894,7 +917,7 @@ def describe_validate_overlapping_fields_can_be_merged():
                 schema=schema,
             )
 
-        def allows_inline_typeless_fragments():
+        def allows_inline_fragments_without_type_condition():
             assert_valid(
                 """
                 {
@@ -931,10 +954,12 @@ def describe_validate_overlapping_fields_can_be_merged():
                 """,
                 [
                     {
-                        "message": fields_conflict_message(
-                            "edges",
-                            [("node", [("id", "name and id are different fields")])],
-                        ),
+                        "message": "Fields 'edges' conflict"
+                        " because subfields 'node' conflict"
+                        " because subfields 'id' conflict"
+                        " because 'name' and 'id' are different fields."
+                        " Use different aliases on the fields"
+                        " to fetch both if this was intentional.",
                         "locations": [
                             (5, 21),
                             (6, 23),
@@ -966,16 +991,6 @@ def describe_validate_overlapping_fields_can_be_merged():
                 schema=schema,
             )
 
-        def error_message_contains_hint_for_alias_conflict():
-            # The error template should end with a hint for the user to try
-            # using different aliases.
-            error = fields_conflict_message("x", "a and b are different fields")
-            assert error == (
-                "Fields 'x' conflict because a and b are different fields."
-                " Use different aliases on the fields to fetch both"
-                " if this was intentional."
-            )
-
         def works_for_field_names_that_are_js_keywords():
             schema_with_keywords = build_schema(
                 """
@@ -1058,9 +1073,10 @@ def describe_validate_overlapping_fields_can_be_merged():
             """,
             [
                 {
-                    "message": fields_conflict_message(
-                        "fido", "name and nickname are different fields"
-                    ),
+                    "message": "Fields 'fido' conflict"
+                    " because 'name' and 'nickname' are different fields."
+                    " Use different aliases on the fields"
+                    " to fetch both if this was intentional.",
                     "locations": [(4, 15), (5, 15)],
                 }
             ],
